// 
//  Copyright (C) 1998-1999,2001,2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H***************************************************************************
 
  * FILE NAME: AsciiWrite.hh
  
  * DEVELOPMENT: Tools
  
  * DESCRIPTION:
  
  This file contains the definition for
 
  * NOTES:
  
        None
  
  * REVISION HISTORY:
  
        Ref. No.        Date
        --------        ----
	2.01            10/22/98


*H****************************************************************************/
#ifndef _ASCIIWRITE_HH_
#define _ASCIIWRITE_HH_


#include <fstream>
#include <vector>

#include "rmf_datafile.hh"


class AsciiWrite : DataFile {
  enum OutputFormat{WIDTH = 8, PRECISION = 2};
public:

  AsciiWrite(DataType type = ASCII):DataFile(type)
  {  d_width = WIDTH; d_precision = PRECISION;};
  virtual ~AsciiWrite(){};

  virtual DataFile::ErrorCode open(const char  *filename, Mode mode = WRITE);
  virtual void close();

  DataFile::ErrorCode write(vector<string> header, short, vector<float> *table);

  DataFile::ErrorCode writeHeader();
  DataFile::ErrorCode writeData();
  DataFile::ErrorCode writeData(char *);
  DataFile::ErrorCode writeData(string str_line){ return writeData(str_line.c_str());};
  DataFile::ErrorCode writeData(vector<string> str_array);

  void putHeader(vector<string> header){ d_lineHeader = header; };
  void putData(short, vector<float> *table);
  void setWidth(short width){d_width = width;};
  void setPrecision(short precision){d_precision = precision;};

  vector<float> getColumn( int colNumb ){ return (d_table[colNumb]);};
  vector<string> getHeader(){ return d_lineHeader;};

  virtual const char *const className() { return ("AsciiWrite"); };

private:

  ofstream        d_outfile;
  vector<float>   *d_table;
  vector<string>  d_lineHeader;
  short           d_numColumns;

  short d_width, d_precision;

};

#endif
