// 
//  Copyright (C) 2005-2009,2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



#ifndef  CALDB4_H_
#define  CALDB4_H_                                                           

#ifdef __cplusplus
extern "C" {
#endif

#ifndef calErrCode 
 #define calErrCode int 
 #define calNOERR   0
 #define calERROR  -1
#endif
#define calBool  int

typedef struct s_calcaldb  calCALDB;  
typedef struct s_calios    calSEARCH; 
  

typedef enum e_calmatch 
  {
    calMATCHAll,  
    calMATCHFirst,    
    calMATCHFirstWarn,
    calMATCHLast, 
    calMATCHLastWarn, 
    calMATCHSingle    
  } calMATCH;

  /* --------------------------------------  */
  
  /*            Function Prototype           */
  
  /* --------------------------------------  */
  
  
  /* initiate  the lib */
  extern calCALDB* calInit(char* tele, char* inst);

  /* set the product */
  extern calSEARCH* calSetProduct(calCALDB*  db_p,  char* val);

  /* set name/value/unit of a parameter */
  extern calErrCode calSetParam(calSEARCH* cal_p, char* nam, char* val, char* unit);
  /*alternately set value for specific params */
  extern calErrCode calSetStartTime(calSEARCH* cal_p, char* val);  
  extern calErrCode calSetStopTime( calSEARCH* cal_p, char* val);  
  extern calErrCode calSetQuality(  calSEARCH* cal_p, char* val);
  extern calErrCode calSetFidelity( calSEARCH* cal_p, char* val);

  /* get the value and unit of the named parameter */
  extern const char* calGetPValue(calSEARCH* cal_p, char* nam);
  extern const char* calGetPUnit( calSEARCH* cal_p, char* nam);

  /* do auto querying */
  extern unsigned int calGetWhichParams(calSEARCH* cal_p);
  extern char* calGetParam(calSEARCH* cal_p, unsigned int indx, calBool col2key);
  extern char* calGetParamUnit(calSEARCH* cal_p, unsigned int indx);

  /* retrieve data */
  extern unsigned int calSearch(calSEARCH* cal_p);

  /* return the result at given index */
  extern char* calGetFile(  calSEARCH* cal_p, unsigned int indx);


  /* hand error */
  extern unsigned long calGetErrNum(calCALDB* db_p);
  extern const char* calGetErrCur( calCALDB* db_p);


  /* close the lib */
  extern void calClose(calCALDB* db_p);


  /* -----------------------
   * utility functions below 
   -------------------------*/

  /* set output Match Methods */
  extern calErrCode calSetMatchMode(calSEARCH* cal_p, calMATCH match);
  /* more way to look up errors */
  extern const char *calGetVerName(void);
  extern const char* calGetWarnCur(calCALDB* db_p);
  extern void calPrintErrCur(calCALDB* db_p);
  extern void calPrintWarnCur(calCALDB* db_p);
  extern void calPrintErrs(calCALDB* db_p);
  extern void calPrintWarns(calCALDB* db_p);

  /* free memory allocated from "char *calGet" functions */
  extern void calFree(char* str);
  /* print outlined "Inputs" on screen */
  extern void calPrintInfo(calSEARCH* cal_p);

  /*get the current CALDB-lib version */
  extern const char *calGetVersion(void);

  /* get extension- name and no of the resulted caldb-file */
  extern const char* calGetFileExtname(calSEARCH* cal_p, unsigned int indx);
  extern short calGetFileExtno(calSEARCH* cal_p, unsigned int indx);
  extern double calGetFidelity( calSEARCH* cal_p, unsigned int indx);
  extern short calGetQuality(  calSEARCH* cal_p, unsigned int indx);

  /*set debug level */
  extern void calSetVerbose(calCALDB* db_p, int);
  extern void calSetErrList(calCALDB* db_p, void*);

#ifdef __cplusplus
}
#endif

#endif /* CALDB4_H_ */
