/*                                                                
**  Copyright (C) 1998-1999,2007  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */



/*************************************************************************

 FILENAME   :  ds_gratings_config.h
 
 DESCRIPTION:  The file ds_gratings_config.h contains defines used by tools
    dealing with gratings data. Included in the defines are items such as the
    TLMIN, TLMAX, and TUNITS of the various columns occurring in gratings
    data files.   

 NOTES      :  The values used in the TLMIN/TLMAX/TUNITS defines are 
    taken from the gratings level 1.5 to archive ICD, rev 1.4 (06/15/98).

*************************************************************************/


#ifndef DS_GRT_CONFIG_H
#define DS_GRT_CONFIG_H


/*  The following defines are intended to centralize TLMIN/TLMAX values
 *  for columns in gratings data files. 
 *
 *  TLMIN/TLMAX - EVENT / REGION 
 */ 

 /* ranges for events */ 
#define DS_GRT_TG_R_TLMIN        -2.0  /* gratings tg_r minimum value   */ 
#define DS_GRT_TG_R_TLMAX         2.0  /* gratings tg_r maximum value   */ 
#define DS_GRT_TG_R_TDBIN    1.593E-4  /* gratings tg_r tdbin value     */ 
#define DS_GRT_TG_D_TLMIN        -2.0  /* gratings tg_d minimum value   */ 
#define DS_GRT_TG_D_TLMAX         2.0  /* gratings tg_d maximum value   */ 
#define DS_GRT_TG_D_TDBIN    1.593E-4  /* gratings tg_d tdbin value     */ 
#define DS_GRT_TG_MLAM_TLMIN   -400.0  /* gratings tg_mlam minimum value*/ 
#define DS_GRT_TG_MLAM_TLMAX    400.0  /* gratings tg_mlam maximum value*/ 
#define DS_GRT_TG_MLAM_TDBIN   2.0E-3  /* gratings tg_mlam tdbin value  */ 
#define DS_GRT_TG_M_TLMIN         -62  /* gratings tg_m minimum value   */ 
#define DS_GRT_TG_M_TLMAX          62  /* gratings tg_m maximum value   */ 
#define DS_GRT_TG_LAM_TLMIN       0.0  /* gratings tg_lam minimum value */ 
#define DS_GRT_TG_LAM_TLMAX     400.0  /* gratings tg_lam maximum value */ 
#define DS_GRT_TG_LAM_TDBIN    2.0E-3  /* gratings tg_lam tdbin value   */ 
#define DS_GRT_TG_PART_TLMIN        0  /* gratings tg_part minimum value*/ 
#define DS_GRT_TG_PART_TLMAX       99  /* gratings tg_part maximum value*/ 
#define DS_GRT_TG_SMAP_TLMIN        0  /* gratings tg_smap minimum value*/ 
#define DS_GRT_TG_SMAP_TLMAX    32767  /* gratings tg_smap maximum value*/ 
#define DS_GRT_TG_SRCID_TLMIN       0  /* gratings tg_srcid minimum val */ 
#define DS_GRT_TG_SRCID_TLMAX   32767  /* gratings tg_srcid maximum val */ 
#define DS_GRT_GDPX_TLMIN           1  /* gratings gdpx minimum value   */ 
#define DS_GRT_GDPX_TLMAX       65535  /* gratings gdpx maximum value   */ 
#define DS_GRT_GDPX_TDBIN          10  /* gratings gdpx tdbin value     */ 
#define DS_GRT_GDPY_TLMIN           1  /* gratings gdpy minimum value   */ 
#define DS_GRT_GDPY_TLMAX       32767  /* gratings gdpy maximum value   */ 
#define DS_GRT_GDPY_TDBIN          10  /* gratings gdpy tdbin value     */ 

/* ranges for regions */
#define DS_GRT_X_TLMIN            0.5  /* region file x column minimum  */ 
#define DS_GRT_X_TLMAX        65534.5  /* region file x column maximum  */ 
#define DS_GRT_Y_TLMIN            0.5  /* region file y column minimum  */ 
#define DS_GRT_Y_TLMAX        65534.5  /* region file y column maximum  */ 
#define DS_GRT_R_TLMIN            0.0  /* region file r column minimum  */ 
#define DS_GRT_R_TLMAX        65534.5  /* region file r column maximum  */ 
#define DS_GRT_ROTANG_TLMIN    -360.0  /* region file rotang minimum    */ 
#define DS_GRT_ROTANG_TLMAX     360.0  /* region file rotang maximum    */ 
#define DS_GRT_COMPONENT_TLMIN      0  /* region file component minimum */
#define DS_GRT_COMPONENT_TLMAX      7  /* region file component maximum */
#define DS_GRT_INCLUDE_TLMIN        0  /* region file component minimum */
#define DS_GRT_INCLUDE_TLMAX        1  /* region file component maximum */
#define DS_GRT_SOURCE_TLMIN         0  /* region file component minimum */
#define DS_GRT_SOURCE_TLMAX     32767  /* region file component maximum */



/*  The following defines are intended to centralize the values of the 
 *  tunit column field values for gratings data files. 
 *
 *  TUNITS - EVENTS/REGIONS
 */

/* units pertaining to events extensions */
#define DS_GRT_TG_RD_TUNITS         "deg"  /* units for event tg_r/d columns */
#define DS_GRT_TG_MLAM_TUNITS  "angstrom"  /* units for event tl_mlam column */
#define DS_GRT_TG_LAM_TUNITS   "angstrom"  /* units for event tg_lam column  */
#define DS_GRT_GDP_TUNITS         "pixel"  /* units for event gdp x/y columns*/
#define DS_GRT_X_TUNITS           "pixel"  /* units for region x column      */
#define DS_GRT_Y_TUNITS           "pixel"  /* units for region y column      */
#define DS_GRT_R_TUNITS           "pixel"  /* units for region r column      */
#define DS_GRT_ROTANG_TUNITS        "deg"  /* units for region rotang column */


/*  The following defines are for L2 grating 'tgextract'.
 *  TLMIN, TLMAX, TDBIN, TNULL  are defined below 
 *
 */
#define DS_TGEX_SPEC_NUM_TLMIN        1   /* spectrum number min */
#define DS_TGEX_SPEC_NUM_TLMAX    16384   /* spectrum number max */
#define DS_TGEX_SPEC_NUM_TNULL        0   /* spectrum number null */

#define DS_TGEX_ROWID_TNULL         NONE  /* spectrum rowid null */

#define DS_TGEX_TG_M_TLMIN          -62   /* spectrum order min */
#define DS_TGEX_TG_M_TLMAX           62   /* spectrum order max */
#define DS_TGEX_TG_M_TNULL           99   /* spectrum order null */

#define DS_TGEX_TG_PART_TLMIN         0   /* spectrum part min */
#define DS_TGEX_TG_PART_TLMAX        99   /* spectrum part max */

#define DS_TGEX_TG_SRCID_TLMIN        1   /* spectrum source id min */
#define DS_TGEX_TG_SRCID_TLMAX    16284   /* spectrum source id max */
#define DS_TGEX_TG_SRCID_TNULL        0   /* spectrum source id null */

#define DS_TGEX_ANCRFILE_TNULL      NONE  /* spectrum ancrfile null */

#define DS_TGEX_RESPFILE_TNULL      NONE  /* spectrum respfile null */

#define DS_TGEX_BACKFILE_TNULL      NONE  /* spectrum background file null */

#define DS_TGEX_CHANNEL_TLMIN        1    /* spectrum channel # min */
#define DS_TGEX_CHANNEL_TLMAX     16384   /* spectrum channel # max */
#define DS_TGEX_CHANNEL_TDBIN        1    /* spectrum channel # bin size */
#define DS_TGEX_CHANNEL_TNULL        0    /* spectrum channel # null */

#define DS_TGEX_COUNTS_TLMIN         0    /* spectrum counts min */
#define DS_TGEX_COUNTS_TLMAX      16384   /* spectrum counts max */
#define DS_TGEX_COUNTS_TDBIN         1    /* spectrum counts bin size */
#define DS_TGEX_COUNTS_TNULL        -1    /* spectrum counts null */

#define DS_TGEX_STAT_ERR_TLMIN      0.0   /* spectrum counts err min */
#define DS_TGEX_STAT_ERR_TLMAX  16384.0   /* spectrum counts err max */
#define DS_TGEX_STAT_ERR_TNULL      NaN   /* spectrum counts err null */

#define DS_TGEX_BACKGROUND_TLMIN     0    /* spectrum bkg counts min */
#define DS_TGEX_BACKGROUND_TLMAX  16384   /* spectrum bkg counts max */
#define DS_TGEX_BACKGROUND_TDBIN     1    /* spectrum bkg counts bin size */
#define DS_TGEX_BACKGROUND_TNULL    -1    /* spectrum bkg counts null */

#define DS_TGEX_BINLO_ANGSTROM_TLMIN     0.0  /* spectrum binlo min */
#define DS_TGEX_BINLO_ANGSTROM_TLMAX   400.0  /* spectrum binlo max */
#define DS_TGEX_BINLO_ANGSTROM_TDBIN  1.0E-4  /* spectrum binlo bin size */
#define DS_TGEX_BINLO_ANGSTROM_TNULL     NaN  /* spectrum binlo null */

#define DS_TGEX_BINLO_DEG_TLMIN    -2.0  /* spectrum binlo min */
#define DS_TGEX_BINLO_DEG_TLMAX     2.0  /* spectrum binlo max */
#define DS_TGEX_BINLO_DEG_TDBIN  8.0E-5  /* spectrum binlo bin size */
#define DS_TGEX_BINLO_DEG_TNULL     NaN  /* spectrum binlo null */

#define DS_TGEX_BINLO_MM_TLMIN  -300.0  /* spectrum binlo min */
#define DS_TGEX_BINLO_MM_TLMAX   300.0  /* spectrum binlo max */
#define DS_TGEX_BINLO_MM_TDBIN   0.012  /* spectrum binlo bin size */
#define DS_TGEX_BINLO_MM_TNULL     NaN  /* spectrum binlo null */

#define DS_TGEX_BINLO_KEV_TLMIN     0.0  /* spectrum binlo min */
#define DS_TGEX_BINLO_KEV_TLMAX    14.0  /* spectrum binlo max */
#define DS_TGEX_BINLO_KEV_TDBIN  1.0E-3  /* spectrum binlo bin size */
#define DS_TGEX_BINLO_KEV_TNULL     NaN  /* spectrum binlo null */

#define DS_TGEX_BINLO_EV_TLMIN      0.0  /* spectrum binlo min */
#define DS_TGEX_BINLO_EV_TLMAX  14000.0  /* spectrum binlo max */
#define DS_TGEX_BINLO_EV_TDBIN      1.0  /* spectrum binlo bin size */
#define DS_TGEX_BINLO_EV_TNULL      NaN  /* spectrum binlo null */

#define DS_TGEX_BINLO_PIX_TLMIN      0  /* spectrum binlo min */
#define DS_TGEX_BINLO_PIX_TLMAX  65536  /* spectrum binlo max */
#define DS_TGEX_BINLO_PIX_TDBIN      1  /* spectrum binlo bin size */
#define DS_TGEX_BINLO_PIX_TNULL      0  /* spectrum binlo null */

#define DS_TGEX_BINHI_ANGSTROM_TLMIN     0.0  /* spectrum binhi min */
#define DS_TGEX_BINHI_ANGSTROM_TLMAX   400.0  /* spectrum binhi max */
#define DS_TGEX_BINHI_ANGSTROM_TDBIN  1.0E-4  /* spectrum binhi bin size */
#define DS_TGEX_BINHI_ANGSTROM_TNULL     NaN  /* spectrum binhi null */

#define DS_TGEX_BINHI_DEG_TLMIN    -2.0  /* spectrum binhi min */
#define DS_TGEX_BINHI_DEG_TLMAX     2.0  /* spectrum binhi max */
#define DS_TGEX_BINHI_DEG_TDBIN  8.0E-5  /* spectrum binhi bin size */
#define DS_TGEX_BINHI_DEG_TNULL     NaN  /* spectrum binhi null */

#define DS_TGEX_BINHI_MM_TLMIN  -300.0  /* spectrum binhi min */
#define DS_TGEX_BINHI_MM_TLMAX   300.0  /* spectrum binhi max */
#define DS_TGEX_BINHI_MM_TDBIN   0.012  /* spectrum binhi bin size */
#define DS_TGEX_BINHI_MM_TNULL     NaN  /* spectrum binhi null */

#define DS_TGEX_BINHI_PIX_TLMIN      0  /* spectrum binhi min */
#define DS_TGEX_BINHI_PIX_TLMAX  65536  /* spectrum binhi max */
#define DS_TGEX_BINHI_PIX_TDBIN      1  /* spectrum binhi bin size */
#define DS_TGEX_BINHI_PIX_TNULL      0  /* spectrum binhi null */

#define DS_TGEX_BINHI_KEV_TLMIN     0.0  /* spectrum binhi min */
#define DS_TGEX_BINHI_KEV_TLMAX    14.0  /* spectrum binhi max */
#define DS_TGEX_BINHI_KEV_TDBIN  1.0E-3  /* spectrum binhi bin size */
#define DS_TGEX_BINHI_KEV_TNULL     NaN  /* spectrum binhi null */

#define DS_TGEX_BINHI_EV_TLMIN      0.0  /* spectrum binhi min */
#define DS_TGEX_BINHI_EV_TLMAX  14000.0  /* spectrum binhi max */
#define DS_TGEX_BINHI_EV_TDBIN      1.0  /* spectrum binhi bin size */
#define DS_TGEX_BINHI_EV_TNULL      NaN  /* spectrum binhi null */

/*  The following defines are for L2 grating 'tgextract'.
 *  TLMIN, TLMAX  are defined below for the region extension
 *
 */
#define DS_TGEX_TG_D_DEG_TLMIN      -2.0   /* region tg_d min */
#define DS_TGEX_TG_D_DEG_TLMAX       2.0   /* region tg_d max */

#define DS_TGEX_TG_D_ASEC_TLMIN  -7200.0   /* region tg_d min */
#define DS_TGEX_TG_D_ASEC_TLMAX   7200.0   /* region tg_d max */

#define DS_TGEX_TG_D_PIX_TLMIN         1   /* region tg_d min */
#define DS_TGEX_TG_D_PIX_TLMAX     32767   /* region tg_d max */

#define DS_TGEX_TG_D_MM_TLMIN     -300.0   /* region tg_d min */
#define DS_TGEX_TG_D_MM_TLMAX      300.0   /* region tg_d max */

#define DS_TGEX_ROTANG_TLMIN      -360.0   /* region rotang min */
#define DS_TGEX_ROTANG_TLMAX       360.0   /* region rotang max */

#define DS_TGEX_COMPONENT_TLMIN      0     /* region component min */
#define DS_TGEX_COMPONENT_TLMAX      7     /* region component max */

#define DS_TGEX_INCLUDE_TLMIN        0     /* region include min */
#define DS_TGEX_INCLUDE_TLMAX        1     /* region include max */


/*  The following defines are for L2 grating 'specdetect'.
 *  TLMIN, TLMAX  are defined below for the output extensions
 *
 */
#define DS_SPECDET_INDEX_TLMIN         1   /* linelist index min */
#define DS_SPECDET_INDEX_TLMAX     16384   /* linelist index max */
#define DS_SPECDET_INDEX_TDBIN         1   /* linelist index bin */
#define DS_SPECDET_INDEX_TNULL         0   /* linelist index null */

#define DS_SPECDET_FWHM_TLMIN        0.0   /* linelist fwhm min */

#define DS_SPECDET_CHANNEL_TLMIN       1   /* spectrum channel min */
#define DS_SPECDET_CHANNEL_TLMAX   16384   /* spectrum channel max */
#define DS_SPECDET_CHANNEL_TDBIN       1   /* spectrum channel bin */
#define DS_SPECDET_CHANNEL_TNULL       0   /* spectrum channel null */


/*  The following defines are for L2 grating 'specmeasure'.
 *  TLMIN, TLMAX  are defined below for the output extensions
 *
 */
#define DS_SPECMEAS_CHISQU_TLMIN     0.0   /* chisqu min */

#define DS_SPECMEAS_NFITBINS_TLMIN     0   /* nfitbins min */
#define DS_SPECMEAS_NFITBINS_TLMAX 16384   /* nfitbins max */
#define DS_SPECMEAS_NFITBINS_TNULL     0   /* nfitbins null */


/*  The following defines are for L2 grating 'specidentify'.
 *  TLMIN, TLMAX  are defined below for the output extensions
 *
 */
#define DS_SPECID_NMATCH_TLMIN         0   /* nmatches min */
#define DS_SPECID_NMATCH_TLMAX     16384   /* nmatches max */
#define DS_SPECID_NMATCH_TNULL         0   /* nmatches null */

#define DS_SPECID_MINDEX_TLMIN         0   /* mindex min */
#define DS_SPECID_MINDEX_TLMAX     16384   /* mindex max */
#define DS_SPECID_MINDEX_TNULL         0   /* mindex null */

#define DS_SPECID_MSTR_TLMIN         0.0   /* mstr min */

#define DS_SPECID_MIONCHG_TLMIN        1   /* mionchg min */
#define DS_SPECID_MIONCHG_TLMAX      100   /* mionchg max */
#define DS_SPECID_MIONCHG_TNULL      999   /* mionchg null */

#define DS_SPECID_MLEV_TLMIN           1   /* lower/upper lev min */
#define DS_SPECID_MLEV_TLMAX         100   /* lower/upper lev max */
#define DS_SPECID_MLEV_TNULL         999   /* lower/upper lev null */


#endif 
