/*                                                                
**  Copyright (C) 1998-1999,2004-2005,2007,2011,2020  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */



#ifndef DS_WCS_H
#define DS_WCS_H

#include <dslib.h>

/* structure containing information needed by the dslib wcs operations */
struct DS_WCS_Data 
{

  dmDescriptor *world[2];	/* descriptor(s) for the world coordinates */
  dmDescriptor *physical[2];	/* descriptors for the physical coordinates */
  char physical_name[2][10];	/* name(s) for the world coordinates */
  char world_name[2][10];	/* name(s) for the physical coordinates */
  char cptname[2][10];		/* name(s) of the word axis elements */
  char cpt2name[2][10];		/* names of the physical axis elements */
  char world_unit[2][dmMAXSTR]; /* world coordinate axis units */
  char phys_unit[2][dmMAXSTR];  /* phys. coordinate axis units */
  char world_trans_type[dmMAXSTR]; /* world coordinates transform type */
  char phys_trans_type[dmMAXSTR];  /* phys. coordinates transform type */
  long axis_groups;		/* how many axis groups */
  long dimension;		/* dimensionality of the axis groups */
  double wcrpix[2], wcrval[2], wcdlt[2];  /* world coordinate parameters */
  double pcrpix[2], pcrval[2], pcdlt[2];  /* physical coordinate parameters */
  double range_min[2], range_max[2]; /* range of data (tlmin/tlmax) */
  double wcs_params[2][4];      /* coordinate transform parameters
				    (not standard wcpix, wcval, wdlt) */
  long nparams[2];              /* number of wcs_params */

  /* offset (in logical pixels) from the center of the logical image
     to the optical axis
     */
  long x_offset, y_offset;	
  float pixel_size;		/* size of the logical pixels (arcsec) */
};

/* 2D Image axis metadata */
struct Image_Axis_Info
{
  long naxes;
  char vector_name[dmMAXSTR];    /* Image axes vector name - if any */
  char axis_names[2][dmMAXSTR];  /* Image axis names                */
  char axis_units[2][dmMAXSTR];  /* Image axis units                */
  double axis_range_min[2];      /* Image axis range minimums       */
  double axis_range_max[2];      /* Image axis range maximums       */

  /* Transform information */
  char   trans_type[dmMAXSTR];   /* Transform type  */
  double crpix[2];               /* Reference pixel */
  double crval[2];               /* Reference value */
  double cdelt[2];               /* Delta per pixel */
  double crota;                  /* Param: CROTA    */
  double equinox;                /* Param: EQUINOX  */
  double lonpole;                /* Param: LONPOLE  */
  double latpole;                /* Param: LATPOLE  */
};
typedef struct Image_Axis_Info  dsImageAxisInfo;

/* Function Prototypes */
extern int ds_coord_calc_d(struct DS_WCS_Data *, double *, double *, double *,
			   dsErrList *);
extern int ds_coord_invert_d(struct DS_WCS_Data *, double *, double *,
			     dsErrList *);
extern int ds_write_wcs_image_keywords(dmDescriptor *, struct DS_WCS_Data *,
				 dsErrList *);

extern int ds_update_wcs_image_keywords(dmDescriptor *, struct DS_WCS_Data *,
				 dsErrList *);

extern int ds_get_wcs_data(dmDescriptor *, float *, long, long,
		     struct DS_WCS_Data *, dsErrList *);

extern int ds_get_image_axis_info(dmDescriptor*, dsImageAxisInfo*, dsImageAxisInfo* );


#endif
