// 
//  Copyright (C) 1998-2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:     @(#)eff_grids.hh

* DEVELOPEMENT:  Chandra X-ray Science Center rmf library
                 hhe@cfa.harvard.edu (initial version)
 
* DESCRIPTION:   Header file
 
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             02/10/98
        0.2             07/10/98
        0.3             08/26/98
 
*H****************************************************************************/

#include "rmf_grids.hh"

//**************************************
//
// class container of EffGrids
//
//**************************************
class EffGrids : public RMFGrids
{

 
public:

  EffGrids();
  EffGrids(dsErrList *_errLst, FILE* _log, short _verb);
  virtual ~EffGrids();

  RMFGeneral::ERROR SetUserGrids(UserGrids *ugrids){
    d_ugrids = ugrids;
    return Success;
  };
  RMFGeneral::ERROR SetFEFGrids(FEFGrids *fgrids){
    d_fgrids = fgrids; 
    return Success;
  };
  RMFGeneral::ERROR SetArray(short _dimn);
  vector<short> GetGriddedAxis(){ return d_fgrids->GetGriddedAxis();};

  RMFGeneral::ERROR FormEffGrids();

  vector<LOCAL_MACROS> *GetGridsFlag(){return d_gridflag; };
  vector<short> *GetCellSize(){return d_cellsize;};
  short GetUserFlagNumber(){return (short)USER_FLAG;};
  float GetFirstGridAtAxis(short axis_no){ return d_grids[axis_no][0];};
  long *ReturnUserGridSize(){return d_ugrids->GetLength();};
  UserGrids *ReturnUserGrids(){return d_ugrids;};

  void Debugging();
  void Fdebugging(short axis);
  void FprintCell(short axis);
  void _PrintArray(const char *name, vector<LOCAL_MACROS> array);
  void _FprintArray(const char *name, vector<LOCAL_MACROS> array);

typedef struct grids_log_s
  {
    short axis_index;
    float eff_grids_range[2];
    float usr_grids_range[2];
    long rm_grids_sz;
  } grids_log;
  grids_log *GetGridsLog(){ return d_grids_log; };

private:

  RMFGeneral::ERROR FindEffFefGridRanges();
  RMFGeneral::ERROR FlagGrids();
  RMFGeneral::ERROR MarkGridCells();
  RMFGeneral::ERROR ConfineFeFGridsToUserGrids();
  //RMFGeneral::ERROR ConfineUserGridsWithinFeFGridBoundary();
  RMFGeneral::ERROR RedefineGridsRange();
  RMFGeneral::ERROR RedefineGridsRange_at(short, vector<float>&);
  void GetEffectiveGridsRange(short axis, float *);

  virtual const char *const className() { return ("EffGrids"); };

  short *d_efgrid_start_index; // effective fef grid starting index 
                               // for each axis
  long  *d_efgrid_length;      // effective fef grid size for each axis
                               // 

  grids_log *d_grids_log;
  vector<LOCAL_MACROS> *d_gridflag;   // flag for user and other grid points
                                      // at each axis
  vector<short> *d_cellsize;          // grid size for each cell for 
                                      // d_dimension number of axis.
                                      // Cell Sides shared by two neighbor 
                                      // Cells are registered to both of them,
                                      // so there is overlaped cell row here.

  //vector<float> *d_ugrids;     // array of User Grids 
  //vector<float> *d_fgrids;     // array of FEF Grids

  UserGrids *d_ugrids;       // pointer to  UserGrids object
  FEFGrids  *d_fgrids;      // pointer to   FEFGrids object

};

