// 
//  Copyright (C) 2006-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:    @(#)fef_keywords.h
 
* DEVELOPEMENT: Chandra X-ray  Science Center fef library
                hhe@cfa.harvard.edu (Initial version)

 
* DESCRIPTION:  Header file
  
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             4/00
 
*H****************************************************************************/

#ifndef _FEF_KEYWORDS_H_
#define _FEF_KEYWORDS_H_

extern "C" {
  #include "ascdm.h"
}

#include "fef_strings.h"

#ifndef FEF_NPOS
#define FEF_NPOS -1
#endif

typedef FefString fstring;

class FEFKeywords 
{
  

public:
  enum {Success = 0, MAX_STRING = 128, MAX_KWD_LNTH=8, MAX_NUM_KEYWDS = 100};

  FEFKeywords(){ SetHeaderDefaults();};
  ~FEFKeywords(){};


  short ChangeKeywordName(fstring keywd, fstring  new_keywd);
  short ChangeComment(fstring keywd, fstring new_comt);
  short ChangeExtension(fstring keywd, fstring new_extn);
  short InsertKeywordAfter(char *keywd, 
			   char *new_keywd,
			   char *new_val,
			   const char *new_typ,
			   char *new_comm,
	  		   char *new_extn);
  short InsertKeywordAfter(long ith,
			   char *new_keywd,
			   char *new_val,
			   char *new_comm,
	  		   char *new_extn);
  short InsertKeywordAfter(long ith,
			   char *new_keywd,
			   char *new_val,
			   const char *new_typ,
			   char *new_comm,
	  		   char *new_extn);
  short InsertKeywordAfter(long ith,
			   char *new_keywd,
			   long new_val,
			   char *new_comm,
			   char *new_extn);
  short InsertKeywordAfter(char *keywd, 
			   char *new_keywd,
			   long new_val,
			   char *new_comm,
			   char *new_extn);
  short InsertKeywordAfter(long ith,
			   char *new_keywd,
			   float new_val,
			   char *new_comm,
			   char *new_extn);
  short InsertKeywordAfter(char *keywd, 
			   char *new_keywd,
			   float new_val,
			   char *new_comm,
			   char *new_extn);
  short SetKeywordValue_f(const char *keywd, float val);
  short SetKeywordValue_l(const char *keywd, long  val);
  short SetKeywordValue_s(const char *keywd, const char *val);

  short SetKeywordValue_f(fstring keywd, float val);
  short SetKeywordValue_l(fstring keywd, long  val);
  short SetKeywordValue_s(fstring keywd, fstring val);

  short GetKeywordArrayIndex(const char *keywd);
  short GetKeywordValue_f(fstring keywd, float *val);
  short GetKeywordValue_l(fstring keywd, long  *val);
  short GetKeywordValue_s(fstring keywd, fstring val);
  short GetKeywordValue_s(const char *keywd, fstring val){
    fstring _keywd;
    _keywd.copy(keywd);
    return GetKeywordValue_s(_keywd, val); }

  // short WriteHeaders( dmBlock* tblBlck, const char* extn);
  long GetNumKeywords();
  void PrtCurrentHeader();
  short GetKeySetAtIndex(long index, 
			 fstring&,fstring&,fstring&,fstring&,fstring&);

  void Replacement_rt(const char *_oldStr, const char *_srcStr);
  void LinkUpdates();

protected:

  fstring 
    d_keywd[MAX_NUM_KEYWDS],
    d_val[MAX_NUM_KEYWDS], 
    d_com[MAX_NUM_KEYWDS], 
    d_extnsn[MAX_NUM_KEYWDS], 
    d_dtype[MAX_NUM_KEYWDS];
 
  char *GetDate_ddmmyyyy(char* a_date);
  char *GetUTCDate_yyyymmdd(char* a_date);
  char *GetUTCTime_hhmmss(char* a_date);
  void Uppercasing();

private:

  long d_numkwd;   // total number of keywords
  void SetHeaderDefaults();
  const char *const className() { return ("FEF_KEYWORDS"); };

};


#endif // _FEF_KEYWORDS_H_




