// 
//  Copyright (C) 2006-2007,2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:    @(#)fef_strings.h
 
* DEVELOPEMENT: Chandra X-ray  Science Center fef library
                hhe@cfa.harvard.edu (Initial version)

 
* DESCRIPTION:  Header file
  
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             4/00
 
*H****************************************************************************/

#ifndef _FEF_STRINGS_H_
#define _FEF_STRINGS_H_

#include <iostream>
#include <sstream>
#include <cstdio>
#include <cctype>

extern "C" {
  #include <string.h>
}

#define FEF_NPOS -1

class FefString {

  enum { len = 256 };

public:

  FefString(){ clear();};
  FefString( const char* _str){  
    if (_str != 0)
      strcpy (str, _str);
    else
      clear();
  };

  ~FefString(){ clear();};
   
  const char *c_str(){return str;};
  
  float str2float() {  
    float val;
    std::istringstream(str ) >> val;
    return val;
  };
  float str2float(char *_str) {  
    float val;
    std::istringstream(_str ) >> val;
    return val;
  };
  float str2float(FefString _str) {  
    float val;
    std::istringstream(_str.str ) >> val;
    return val;
  };
 
 long  str2long(){
    long val;
    std::istringstream(str ) >> val;
    return val;
  };
  long  str2long(char *_str) {   
    long val;
    std::istringstream(_str ) >> val;
    return val;
  };

  int find(FefString _str) {
    if( strstr(str, _str.str) )
      return 1; // Found 
    else
      return FEF_NPOS; // not Found
  };	  
    
  long length(){ return strlen(str);};
  void copy(const char* _str){clear(); sprintf(str,"%s", _str);    };
  void copy(FefString _str){clear(); sprintf(str,"%s", _str.str);};

  void append(const char* _str){sprintf(str,"%s%s", str, _str);    };
  void append(FefString _str){sprintf(str,"%s%s", str, _str.str);};
  void preappend(char* _str){
    char tmp[len]; 
    strcpy(tmp, str); 
    sprintf(str,"%s%s", _str, tmp);
  };
  void preappend(FefString _str){
    char tmp[len]; 
    strcpy(tmp, str); 
    sprintf(str,"%s%s", _str.str, tmp);
  };

  void push_back(const char* _str){ sprintf(str, "%s", _str);};

  void upper_case(){
    char *upr_case = new char [strlen(str) +1];

    strcpy(upr_case, str );
    for(short ii = 0; ii < (short)strlen(str); ii++) {
      if( isalpha(upr_case[ii]) ) {
	if( islower(upr_case[ii]) )
	  upr_case[ii] = upr_case[ii] - 'a' + 'A';
      }
    }
    strcpy(str, upr_case); 
    delete [] upr_case;
  };

  void upper_case(char *_str){
    strcpy(str, _str );
    upper_case();
    strcpy(_str, str );
  };
  /*
   * Get Substring sets *
   */
  char *get_substring(char* _ldelim, char *_rdelim){
    return get_substring(str, _ldelim, _rdelim);
  }

  char *get_substring(const char *_src, char* _ldelim, char *_rdelim){
    return get_substring((char *)_src, _ldelim, _rdelim);
  }

  char *get_substring(char *_src, char* _ldelim, char *_rdelim){
    if(strlen(_src) == 0) return NULL;

    char tmpstr[len];
    for(long ii = 0; ii < len; ii++) tmpstr[ii]='\0';
    char *beg, *end;
    beg = strstr(_src, _ldelim);
    end = strstr(_src, _rdelim);

    if(beg == NULL && end != NULL)
      strncpy(tmpstr, _src,   (strlen(_src) - strlen(end) ) );
    else if(beg != NULL && end == NULL) {
      /* point to  pos immed after the first part of _ldelim */
      beg += strlen(_ldelim);
      strcpy(tmpstr, beg );
    } else {
      beg += strlen(_ldelim);
      /* total chars within the d_ldelim and d_rdelim */
      /* extract substr inside, say, a brackets "[]", not include the brackets */
      strncpy(tmpstr, beg,   (strlen(beg) - strlen(end) ) );
    }
    clear();
    strcpy(str, tmpstr);
    return &str[0];
  }

  char *get_substring_last(char* _delim){
    strcpy(str, get_substring_last( str, _delim) );
    return &str[0];
  }

  char *get_substring_last(const char *_src, char* _delim){
    return get_substring_last( (char *)_src, _delim);
  }
  char *get_substring_last(char *_src, char* _delim){
    if(strlen(_src) == 0) return NULL;

    char *_ptr;
    if( (_ptr = strstr(_src, _delim) )  == NULL ) 
      return _src;

    /* point to  pos immed after the first part of _delim */
    _ptr += strlen(_delim);

    /* extract substr inside, say, a brackets "[]", not include the brackets */
    char tmpstr[len];
    for(long ii = 0; ii < len; ii++) tmpstr[ii]='\0';
    strcpy(tmpstr, _ptr);

    clear();
    strcpy(str, tmpstr); 

    return &str[0];
  }
  char *get_substring_first(char* _delim){
    strcpy(str, get_substring_first( str, _delim) );
    return &str[0];
  }

  char *get_substring_first(const char *_src, char* _delim){
    return get_substring_first( (char *)_src, _delim);
  }
  char *get_substring_first(char *_src, char* _delim){
    if(strlen(_src) == 0) return NULL;

    char *_ptr;
    if( (_ptr = strstr(_src, _delim) )  == NULL ) 
      return _src;

    /* total chars within the d_ldelim and d_rdelim */
    /* extract substr inside, say, a brackets "[]", not include the brackets */
    char tmpstr[len];
    for(long ii = 0; ii < len; ii++) tmpstr[ii]='\0';
    strncpy(tmpstr, _src, (strlen(_src) - strlen(_ptr)) );

    clear();
    strcpy(str, tmpstr);
    return &str[0];
  }

  void clear(){ for(long ii = 0; ii < len; ii++) str[ii]='\0';}
  void replace_first(FefString _str, const char *_oldStr, const char *_newStr)
  {
    strcpy(str, _str.c_str() );
    replace_first( _oldStr, _newStr);
  }
  void replace_first( const char *_oldStr, const char *_newStr)
  {
    long ii;
    size_t n;
    char _sstr[len];

    _sstr[0]='\0';
    char *pt = strstr(str, _oldStr);
    if( !pt )
      return;

    n = 0;
    for(ii=0; ii < (long)strlen(str); ii++) {
      if(&str[ii] == pt) {
	strncpy(_sstr, str, n);
	_sstr[n]='\0';
	break;
      }
      n++;
    }
    strcat(_sstr, _newStr);
    for(ii=0; ii < (long)strlen(_oldStr); ii++) 
      pt++;
    strcat(_sstr, pt);
  
    strcpy(str, _sstr);
  }

private:
  char str[len];
 
};

#endif // _FEF_STRINGS_H_
