// 
//  Copyright (C) 2003-2008,2010  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

/*JCC(2/2003)- initial version */
/*JCC(7/2003)- support dmSHORT dmUSHORT dmULONG */
/*JCC(4/2010)- add hdrPTR */
// 8/2010 - add hdrFreeKeyValue_c
#ifndef HDRLIB2_H
#define HDRLIB2_H

#ifndef HDR_STATUS_H
#include "hdr_status.h"
#endif


/* creator */
#define  TOOL_DEF         "cxc"  /* creator="cxc - Version $SCIDEV_VERSION2" */

/* ascdsver */
#ifndef ASCDSNUM
#define ASCDSNUM  "ASCDS version number"
#endif

/*------------------
 *    constants
 *------------------*/
#define  HDR_SZ       511

typedef enum {hdrPAR_FILE=1, hdrDM_FILE=2, hdrCFITS_FILE=3}    hdrFILE ;
typedef enum {hdrDOUBLE=1,hdrCHAR,hdrLONG,hdrULONG,hdrBOOL,hdrPTR,hdrUNKNOWN} hdrDTYPE;
typedef enum {hdrNOT_FOUND=0, hdrFOUND=1}  hdrSEARCH ;

/* 0:putHdr won't write it; if key exists in outBlock, delete it ; (for hdr merge) 
 * 1:putHdr write it (or overwrite if key exists in outBlock) 
 * 2:putHdr won't write it; if key exists in outBlock, keep it; (for hpe) 
 * 3:only for HDUCLASS,HDUCLAS1,HDUCLAS2,HDUCLAS3; 
 *       putHdr won't write them unless mergeHdr is called; If no
 *       merging rule specified in ltable, then the 1st value will
 *       be copied.
 */
typedef enum {hdrKFLAG_0=0, hdrKFLAG_1=1, hdrKFLAG_2=2, hdrKFLAG_3=3}   hdrFLAG ;

typedef  unsigned long     BASIC_STATUS_T ;

/*---------------------
 * function prototype
 *--------------------*/
#ifdef __cplusplus    /* hdr_basic.hh */
extern "C"
{
#endif
  typedef struct _Header_Type Header_Type;

  extern Header_Type *getHdr(void *inblock,  int file_type);
  extern int putHdr(void *outBlock, int file_type, Header_Type *head,
                      unsigned long key_to_put, char * toolname);
  extern Header_Type *mergeHdr(char *ltable, Header_Type **mArr, int nfiles);
  extern Header_Type *buildHdr(int file_type, unsigned long sts);

  /* in hdrFunc.cc  */
  extern int  hdrFindKey(Header_Type *kmp, const char *name);

  extern int hdrGetKeyValue_d(Header_Type *kmp, const char *name, double *dd);
  extern int hdrGetKeyValue_l(Header_Type *kmp, const char *name, long *dd);
  extern int hdrGetKeyValue_ul(Header_Type *kmp, const char *name, unsigned long *dd);
  extern int hdrGetKeyValue_b(Header_Type *kmp, const char *name, short *dd);
  extern int hdrGetKeyValue_c(Header_Type *kmp, const char *name, char **dd);
  extern hdrDTYPE hdrGetKeyType(Header_Type *kmp, const char *name);
  extern int hdrGetKeyComm(Header_Type *kmp, const char *name, char **dd);
  extern int hdrGetKeyUnit(Header_Type *kmp, const char *name, char **dd);

  extern int hdrSetKeyValue_d(Header_Type *kmp, const char *name, double dd);
  extern int hdrSetKeyValue_l(Header_Type *kmp, const char *name, long dd);
  extern int hdrSetKeyValue_ul(Header_Type *kmp, const char *name, unsigned long dd);
  extern int hdrSetKeyValue_b(Header_Type *kmp, const char *name, short dd);
  extern int hdrSetKeyValue_c(Header_Type *kmp, const char *name, const char * dd);
  extern int hdrSetKeyComm(Header_Type *kmp, const char *name, const char * dd);
  extern int hdrSetKeyUnit(Header_Type *kmp, const char *name, const char * dd);
  extern int hdrSetKeyFlag(Header_Type *kmp, const char *name, int dd);
  extern int hdrSetKeyStatus(Header_Type *kmp, const char *name, unsigned long dd);

  extern int hdrSetKey_d(Header_Type *kmp, const char *name, double dd, const char *comm, const char *unit);
  extern int hdrSetKey_l(Header_Type *kmp, const char *name, long dd, const char *comm, const char *unit);
  extern int hdrSetKey_ul(Header_Type *kmp, const char *name, unsigned long dd, const char *comm, const char *unit);
  extern int hdrSetKey_b(Header_Type *kmp, const char *name, short dd, const char *comm, const char *unit);
  extern int hdrSetKey_c(Header_Type *kmp, const char *name, const char *dd, const char *comm, const char *unit);

  extern void hdrFreeKeyValue_c (char *cc) ;

   /* --- misc --- */
   extern void hdrUpdateDateObsEnd(Header_Type *kmp);
   extern void freeHdr( Header_Type *hdr) ;

#ifdef __cplusplus    /* hdr_basic.hh */
}
#endif

#endif
