/*                                                                
**  Copyright (C) 1970,1996-2004,2007,2014-2015  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*H***************************************************************************
 
  * FILE NAME: pixlib.h
  
  * DEVELOPMENT: Tools
  * DESCRIPTION:
  
  This file contains the definition for
 
  * NOTES:

  * REVISION HISTORY:
  
        Ref. No.        Date
        --------        ----
         0.1            9/1995

*H****************************************************************************/
#ifndef _PIXLIB_H_
#define _PIXLIB_H_

#ifdef  __cplusplus
extern "C" {
#endif

# include <bool.h> 
# include <stdarg.h>
# include "ascdm.h"

  /*********************************************\
 |     Dimension Type definitions                |
 |  (Followed are wrapped into pix_dimndata.h)   |
  \*********************************************/


#ifndef  PIX_SZ_STRING
#define  PIX_SZ_STRING 1023                 /* long size of a string */
#endif

#ifndef PIX_GOOD
#define PIX_GOOD      0                 /* return status  of success */
#endif


#ifndef _PIX_UTILS_H_
  typedef double VEC2_DBLE[2];          /* 2-d double data type       */
  typedef double VEC3_DBLE[3];          /* 3-d double data type       */
  typedef double VEC6_DBLE[6];          /* 6-d double data type       */
  typedef long   VEC2_LONG[2];          /* 2-d long integer data type */ 
  typedef short  VEC2_SHRT[2];          /* 2-d short integer data type*/ 
  typedef char  VECL_CHAR[PIX_SZ_STRING];  /* long char data type        */
  typedef char  VECS_CHAR[PIX_SZ_STRING];  /* short char data type       */
#endif
  typedef double M4x2_DBLE[4][2];       /* 4x2 double data type       */


char *pix_get_parfile_string(void);
char *pix_get_version_string(void);


short pix_write_history(short (*fun)(void *fp, char *buf), void *fpt);
short pix_write_params( short (*fun)(void *fp, char *buf), void *fpt);
short pix_write_config( short (*fun)(void *fp, char *buf), void *fpt);
void pix_clean_history_stack(void);

void pix_error (char *fmt, ...);
void pix_message (int level, char *fmt, ...);
 int pix_set_verbosity (int verbosity);

 /******************************\
|    Iniailization and Closing   |
 \******************************/
/*
 Starts pixlib system structuring and load up necessary database 
 (*.par files). Set up system defaults for the given telescope
 Valid "telescope" includes 
	"CHANDRA" and "XRCF".
 Each process is designed  on one telescope per event. To choose
 another telescope, you have to initialize it again after close it.
 Must be called first always.

 Returns 0 for success and others for failure"
 */
 /* short pix_init_pixlib(const char *telescope, ...); */
short pix_init_pixlib(const char *telescope, const char *parfile);

/*
 To end an event for a specified telescope  in order to free the memory 
 previously allocated by pix_init_pixlib(...).
 It must be executed before exit pixlib.
 */
void pix_close_pixlib(void);


 /*******************************\
|     Debug and Help Calls        |
 \*******************************/
/*
  Standard output of system wide information to provide user awareness.
 */

bool pix_is_init(void);        /* return TRUE if pix_init_pixlib() is called */
void pix_disp_detector(void);  /* display all valid       DETector names   */
void pix_disp_tdetsys(void);   /* display all valid         TDET systems   */
void pix_disp_fpsys(void);     /* display all valid           FP systems   */
void pix_disp_gdpsys(void);    /* display all valid          GDP systems   */
void pix_disp_chip(void);      /* display all valid  CHIP ID and names     */
void pix_disp_aimpoint(void);  /* display all valid aim-point ID and names */
void pix_disp_grating(void);   /* display grating properties  */

 /****************************\
|     System Configuration     |
 \****************************/

/* 
   Set individual system components to override default system configuration
   for given inputs. All "set" actions will taken for the current observation 
   telescope: AXAF or XRCF.

   It shall be user's responsibility to manage any  memory which might be 
   allocated for those applications below.

   The valid inputs can be found in "Pixlib User's Guide" (To Be Available).
   Most of function names are self informative for their functionality.

   Returns 0 for success and others for failure
  */

/* set detector sim-offset, and default aimpoint */
short pix_set_detector(const char *detector);

/* set TDET system for the current detector */			
short pix_set_tdetsys(const char *tdet_sys);
                        
short pix_set_fpsys(  const char *fp_sys); 
short pix_set_gdpsys( const char *gdp_sys);
short pix_set_grating(const char *grt_arm);
short pix_set_shell(const char *shell);
short pix_set_shell_id(short id);
short pix_set_grating_order( short grt_order);
short pix_set_aimpoint(const char *aimpoint);
short pix_set_aimpoint_by_value( VEC3_DBLE aimpnt_pos);

short pix_set_gzo(VEC3_DBLE fc_zo); 

short pix_set_align(  VEC3_DBLE mirror_align, VEC3_DBLE stage_align);
short pix_set_mirror( VEC2_DBLE mpy, VEC3_DBLE stage, VEC3_DBLE stage_ang);
void  pix_set_randseed(unsigned long seed_flag);

/* 
  set all defaults, including tdetsys, fpsys, gdpsys, grating_arm, 
  grating_order, aimpoint for the current telescope and detector. 
 */
short pix_set_pixlib_defaults(void);

/* set all defaults depedent upon telescope */
short pix_set_pixlib_defaults_telescope(void);

/* set all defaults depedent upon detector */
short pix_set_pixlib_defaults_detector(void);

short pix_set_aimpoint_default(void);
short pix_set_fpsys_default(void);
short pix_set_gdpsys_default(void);
short pix_set_grtarm_default(void); 
short pix_set_grtorder_default(void); 
short pix_set_tdetsys_default(void);

                   

 /*****************************************\
|   Return State of System Configuration    |
 \*****************************************/
/*
   Get individual component names or quantities of the current system.

   Returns 0 for success and others for failure.
 */

short pix_get_chipsys_default(VECL_CHAR chip_sys);
short pix_get_tdetsys_default(VECL_CHAR tdet_sys);
short pix_get_fpsys_default(VECL_CHAR fp_sys );
short pix_get_aimpoint_default(VECS_CHAR aim_point);
short pix_get_grtarm_default(VECS_CHAR grt_arm);
short pix_get_gdpsys_default(VECL_CHAR gdp_sys);
short pix_get_shell_id(void);
short pix_get_shell_at_id(short id, VECS_CHAR  shell_name);
short *pix_get_shell_ids(short *num, short **ids);
short pix_get_tgdiam(double *tgdiam);
short pix_get_tgdiam_at_id(short id, double *tgdiam);

short pix_get_detector_name(VECS_CHAR detector);
short pix_get_chip_name(short chip_id /* input */, VECS_CHAR chip_name);
short pix_get_aimpoint_name(VECS_CHAR aim_point);
short pix_get_aimpoint_by_value( VEC3_DBLE aimpnt_pos);
short pix_get_no_chips(void);

short pix_get_tdetsys_name(VECL_CHAR tdet_sys);
short pix_get_gdpsys_name(VECL_CHAR  gdp_sys);
short pix_get_grtarm_name(VECS_CHAR grt_arm);
short pix_get_fpsys_name(VECL_CHAR fp_sys);
short pix_get_telescope_name(VECS_CHAR telescope);
                           
             
short pix_get_chip_id0_and_num(VECS_CHAR chip_name, short *, short*);
short pix_get_chip_id(VECS_CHAR chip_name);
short pix_get_detector_id(void);
short pix_get_aimpoint_id(void);
short pix_get_grtarm_id(void);
short pix_get_tdetsys_id(void);
short pix_get_fpsys_id(void);
short pix_get_gdpsys_id(void);

short pix_get_canchip_name(VECS_CHAR chip_name, VECS_CHAR canchip_name);
short pix_get_detchip_name (VECS_CHAR chip_name, VECS_CHAR detchip_name);

short pix_map_aimpoint(VECS_CHAR aim_point, VEC3_DBLE aimpoint_value);

/* Get the current telescope focal length in mm */
short pix_get_flength(double *flength); 

/* Get Rowland circle Radius in mm */		                  
short pix_get_rowland( double *rowland);
/* Get OTG origin im mm */
short pix_get_OTG_origin(VEC3_DBLE g0);

/* 
  Get the image axis type, short or long, for the specified system 
  (TDET, FP, or GDP)  */

short pix_get_image_axis_type(const char* sys, VEC2_SHRT axis_type);	
	          
/* 
 Get chip scale in mm/pixel, chip size in pixel
 for the specified chip ID of the current detector
 */
short pix_get_chip_corner_at_fpc(short chip_id, M4x2_DBLE corner);
short pix_get_chip_scale_in_mm(VEC2_DBLE scale);
short pix_get_chip_scale_in_asec(double *scale);
short pix_get_chip_size(short chip_id,  VEC2_LONG size);


/*
  Get the center, dimension of the image plane in pixels for the 
  current systems (TDET, FP, and GDP). 
  Get the scalar in mm/pixel or arc-sec/pixel, or angstrom of the current systems 
  (TDET, FP, and GDP).
 */
short pix_get_tdet_image_cntr(    VEC2_DBLE tdet_cntr);
short pix_get_tdet_image_dimn(    VEC2_LONG tdet_dimn);

short pix_get_fp_image_cntr(      VEC2_DBLE ima_cntr);
short pix_get_fp_image_dimn(      VEC2_LONG ima_dimn);
short pix_get_fp_scale_in_mm(     double *scale);
short pix_get_fp_scale_in_asec(   double *scale);


short pix_get_gdp_image_cntr(   VEC2_DBLE ima_cntr);
short pix_get_gdp_image_dimn(   VEC2_LONG ima_dimn);
short pix_get_gdp_scale_in_asec(     double *scale);
short pix_get_gdp_scale_in_angstrom( double *scale);


 /*****************************************\
|          Overall Transformation           |
 \*****************************************/

/*
   Transform coordinates <a> to <b> within the current system configuration
   (telescope, detector, aimpoint, FP, GDP, TDET, etc.)
   Returns 0 for success and others for failure.

   Note:
     "chip_id" argument in pix_chip_to_<foo>(...) will be an input (short type),
     but in pix_<foo>_to_chip(...) will be output, or a pointer to short type.
 */

short pix_chip_to_mpy(short chip_id, VEC2_DBLE chip, VEC2_DBLE mpy);
short pix_chip_to_mnc(short chip_id, VEC2_DBLE chip, VEC3_DBLE mnc);
short pix_chip_to_fpc(short chip_id, VEC2_DBLE chip, VEC2_DBLE fpc);
short pix_extended_chip_to_fpc(short chip_id, VEC2_DBLE chip, VEC2_DBLE fpc);  
short pix_chip_to_fc( short chip_id, VEC2_DBLE chip, VEC3_DBLE fc);
short pix_chip_to_gac(short chip_id, VEC2_DBLE chip, VEC2_DBLE gac);
short pix_fpc_to_chip(VEC2_DBLE fpc, short *chip_id, VEC2_DBLE chip);
short pix_fpc_to_extended_chip(VEC2_DBLE fpc, short chip_id, VEC2_DBLE chip);
short pix_fpc_to_extended_chip_and_id(VEC2_DBLE fpc, short *chip_id, VEC2_DBLE chip);
short pix_fpc_to_cpc( VEC2_DBLE fpc, short *chip_id, VEC3_DBLE cpc);
short pix_lsi_to_fc( VEC3_DBLE lsi, VEC3_DBLE fc);
short pix_fc_to_lsi( VEC3_DBLE fc,  VEC3_DBLE lsi);
short pix_fpc_to_gdp(VEC2_DBLE fpc, VEC2_DBLE gdp);
short pix_fc_to_gdp( VEC3_DBLE fc,  VEC2_DBLE gdp);
short pix_mpy_to_fpc( VEC2_DBLE mpy,  VEC2_DBLE fpc);
short pix_ray_to_detector(VEC3_DBLE mnc, VEC3_DBLE mnc_dir, 
			      short *chip_id, VEC2_DBLE chip, VEC3_DBLE lsi);

 /************************************************\
|            Grating Transformation                |
 \************************************************/
/*
  Make grating coordinate transformation within the current system configuration
  (grating arm, grating order, GDP system)
  Returns 0 for success and others for failure
 */

/* Get FC at zero order for the specified mpy */
short pix_get_zo(   VEC2_DBLE mpy, VEC3_DBLE fc_zo);

/* Convert distance in mm to angular one in degree */
short pix_get_sky_angle(VEC2_DBLE gac /* input */, VEC2_DBLE sky_ang);
/*
short pix_gac_to_gwc(VEC2_DBLE gac, double *gwc);
short pix_gac_to_eng(VEC2_DBLE gac, double *eng );
short pix_eng_to_gac(double eng,   VEC2_DBLE gac);
*/

short pix_get_energy(VEC2_DBLE gac, double *energy );
short pix_get_grating_wavelength(VEC2_DBLE gac, double *gwc);
short pix_get_grating_period( double *period);
short pix_get_grating_angle( double *angle);
short pix_get_grating_property( VEC2_DBLE prop);

short pix_mnc_to_gnc(VEC3_DBLE mnc, VEC3_DBLE gnc);
short pix_gnc_to_mnc(VEC3_DBLE gnc, VEC3_DBLE mnc);
short pix_mnc_to_gzo(VEC3_DBLE mnc, VEC3_DBLE gzo);
short pix_gzo_to_mnc(VEC3_DBLE gzo, VEC3_DBLE mnc);
short pix_gzo_to_gac(VEC3_DBLE gzo, VEC2_DBLE gac);
short pix_gac_to_gdc(VEC2_DBLE gac, VEC2_DBLE gdc);
short pix_gzo_to_gdp(VEC3_DBLE gzo, VEC2_DBLE gdp);
short pix_gac_to_gdp(VEC2_DBLE gac, VEC2_DBLE gdp);
short pix_gdp_to_gac(VEC2_DBLE gdp, VEC2_DBLE gac);
short pix_gdp_to_gzo(VEC2_DBLE gdp, VEC3_DBLE gzo);
short pix_gac_to_gzo(VEC2_DBLE gac, VEC3_DBLE gzo);
short pix_gac_to_gdc(VEC2_DBLE gac, VEC2_DBLE gdc);
short pix_fc_to_gzo( VEC3_DBLE fc,  VEC3_DBLE  gzo);
short pix_gzo_to_fc( VEC3_DBLE gzo, VEC3_DBLE  fc );



 /********************************************\
|            FAM  Coordinates System           |
 \********************************************/

/*
  Make FAM coordinates transformation within the current system configuration
  (mirror configuration and alignment and FP system ).
  Returns 0 for success and others for failure
 */

short pix_stf_to_dfc(VEC3_DBLE stf, VEC3_DBLE dfc);
short pix_dfc_to_stf(VEC3_DBLE dfc, VEC3_DBLE stf);
short pix_mnc_to_dfc(VEC3_DBLE mnc, VEC3_DBLE dfc);
short pix_dfc_to_mnc(VEC3_DBLE dfc, VEC3_DBLE mnc);
short pix_stf_to_mnc(    VEC3_DBLE stf, VEC3_DBLE mnc);
short pix_mnc_to_stf(    VEC3_DBLE mnc, VEC3_DBLE stf);


 /**********************************************\
|    Tiled Detector Pixel  Coordinates System    |
 \**********************************************/

/*
  Make TDET coordinates transformation within  the current TDET and detector.
  Returns 0 for success and others for failure
 */

short pix_tdet_to_chip(  VEC2_DBLE tdet, short *chip_id, VEC2_DBLE chip);
short pix_chip_to_tdet(  short chip_id,  VEC2_DBLE chip, VEC2_DBLE tdet);

/* 
 Get chip range (minimum or maximum) in current TDET plane system
 Returns 0 for success and others for failure
*/

short pix_get_tdet_chip_min( VEC2_DBLE chip_min);
short pix_get_tdet_chip_max( VEC2_DBLE chip_max);




 /**********************************************\
|        CHIP-STF Coordinates Conversion         |
 \**********************************************/

/*
  Make transformation among CHIP-LSI-STF coordinate within the current
  detector and  aimpoint.
  Returns 0 for success and others for failure
 */

short pix_cpc_to_chip(VEC3_DBLE cpc, VEC2_DBLE chip);
short pix_chip_to_lsi(short chip_id, VEC2_DBLE chip, VEC3_DBLE lsi);
short pix_stf_to_chip(short chip_id, VEC3_DBLE stf,  VEC2_DBLE chip);
short pix_stf_to_lsi( VEC3_DBLE stf,  VEC3_DBLE lsi);
short pix_lsi_to_stf( VEC3_DBLE lsi,  VEC3_DBLE stf);
short pix_lsi_to_mnc( VEC3_DBLE lsi, VEC3_DBLE mnc);
short pix_mnc_to_lsi( VEC3_DBLE mnc, VEC3_DBLE lsi);


 /**********************************************\
|        STF - TPC  Coordinates Conversion       |
 \**********************************************/
/*
   Make coordinate transformation among STF and TPC for the current 
   Detector, FP, mirror configuration and alignment.
   Returns 0 for success and others for failure
 */


short pix_fc_to_fpc(     VEC3_DBLE fc,  VEC2_DBLE fpc);
short pix_fpc_to_fc(     VEC2_DBLE fpc, VEC3_DBLE fc);
short pix_fc_to_mnc(     VEC3_DBLE fc,  VEC3_DBLE mnc);
short pix_mnc_to_fc(     VEC3_DBLE mnc, VEC3_DBLE fc );
short pix_fc_to_stf(     VEC3_DBLE fc,  VEC3_DBLE stf);
short pix_stf_to_fc(     VEC3_DBLE stf, VEC3_DBLE fc );
short pix_stf_to_mnc(    VEC3_DBLE stf, VEC3_DBLE mnc);
short pix_mnc_to_stf(    VEC3_DBLE mnc, VEC3_DBLE stf);
short pix_mnc_to_fpc(    VEC3_DBLE mnc, VEC2_DBLE fpc); 
short pix_fpc_to_mnc(    VEC2_DBLE fpc, VEC3_DBLE mnc);
short pix_fpc_to_mnc_dir(VEC2_DBLE fpc, VEC3_DBLE mnc);



 /**********************************************\
|      TPC - SKY  Coordinates  Conversion        |
 \**********************************************/
/*
  Generic angular  coordinates transformation.
  Returns 0 for success and others for failure
 */

short pix_mnc_to_hrma( VEC3_DBLE mnc,  VEC3_DBLE hrma);
short pix_hrma_to_mnc( VEC3_DBLE hrma,  VEC3_DBLE mnc);

short pix_msc_to_mnc( VEC3_DBLE msc,  VEC3_DBLE mnc);
short pix_mnc_to_msc( VEC3_DBLE mnc,  VEC3_DBLE msc);
short pix_msc_rh_to_mnc( VEC3_DBLE msc,  VEC3_DBLE mnc);
short pix_mnc_to_msc_rh( VEC3_DBLE mnc,  VEC3_DBLE msc);
short pix_msc_rh_to_mpy( VEC3_DBLE msc,  VEC2_DBLE mpy);
short pix_fsc_to_mnc(VEC3_DBLE fsc,  VEC3_DBLE mnc);
short pix_mnc_to_fsc(VEC3_DBLE mnc,  VEC3_DBLE fsc);
short pix_fsc_to_msc(VEC3_DBLE fsc,  VEC3_DBLE msc);
short pix_msc_to_fsc(VEC3_DBLE msc,  VEC3_DBLE fsc);
short pix_mnc_to_mpy( VEC3_DBLE mnc,  VEC2_DBLE mpy);
short pix_mpy_to_mnc( VEC2_DBLE mpy,  VEC3_DBLE mnc);
short pix_mpy_to_msc( VEC2_DBLE mpy,  VEC3_DBLE msc);
short pix_msc_to_mpy( VEC3_DBLE msc,  VEC2_DBLE mpy);


 /******************************\
|       Aspect Solution          |
 \******************************/

/*
  Coordinate transformation between SKY and FPC (det) tangent plane systems.
  SKY: bearing aspect effects.
  FPC: corrected with aspect offsets.
  Returns 0 for success and others for failure
 */

/* convert detector plane coordinate (or FPC) to SKY (add aspect offset )*/
short pix_apply_aspect(  VEC2_DBLE det, VEC3_DBLE asp_offset, VEC2_DBLE sky);

/* convert SKY to detector plane coordinate (or FPC) (remove aspect offset )*/
short pix_deapply_aspect(VEC2_DBLE sky, VEC2_DBLE asp_offset, VEC2_DBLE det);


/* get absolute aspect for given aspect pixel offset,  nominal aspect, and scale */
short pix_get_asp_absolute(VEC3_DBLE asp_offset, VEC3_DBLE nom_asp, 
			  double scale, VEC3_DBLE asp );

/* get absolute aspect for given aspect sky offset,  absolute aspect, & scale*/
short pix_get_asp_nominal(VEC3_DBLE sky_offset, VEC3_DBLE asp, 
			  double scale, VEC3_DBLE nom_asp );

/* get aspect offsets for given nominal aspect, absolute aspect, and scale */
short pix_get_asp_offset(VEC3_DBLE nom_asp, VEC3_DBLE asp, double scale, 
			 VEC3_DBLE asp_offset);

/* get detector offsets for given aspect offset and   absolute aspect */
short pix_get_det_offset(VEC3_DBLE asp_offset, double roll,
			 VEC3_DBLE det_offset);

/* get sky offsets for given nominal aspect offsets and  absolute aspect */
short pix_get_sky_offset(VEC3_DBLE nom_asp, VEC3_DBLE asp, double scale, 
                         VEC3_DBLE sky_offset);
			 
 /****************************************\
|       Telemetry  Data  Conversion        |
 \****************************************/

/*
  HRC telemetry and chip data conversion.
  Returns 0 for success and others for failure
 */

short pix_coarse_to_dgp( VEC3_DBLE u_coarse, VEC3_DBLE v_coarse,
			 short *chip_id,   VEC2_DBLE dgp );
short pix_coarse_to_raw( VEC3_DBLE u_coarse, VEC3_DBLE v_coarse,
                          short *chip_id,  VEC2_DBLE raw     );

short pix_tap_to_tel( VEC2_DBLE tap, short *chip_id,   VEC2_DBLE tel );
short pix_tap_to_chip(VEC2_DBLE tap, short *chip_id,   VEC2_DBLE chip );
short pix_chip_to_tap(short chip_id, VEC2_DBLE tap,    VEC2_DBLE chip );
short pix_wchip_to_chip(short wsize, double cy0, VEC2_DBLE wchip, VEC2_DBLE chip);


/*******************************************\
|        Pixel Data Type Conversions         |
\*******************************************/

/*
   pixel conversion among type long, shot, double, and float.
   Given random_around = TRUE, the rounding decimal will be generated
   through randomization, or a fixed decimal is choose.
   Returns 0 for success and others for failure
 */

short pix_long_to_double(   long pos_in,  bool random_round, double *pos_out);
short pix_short_to_double( short pos_in,  bool random_round, double *pos_out);
short pix_double_to_short(double pos_in,  bool random_round, short  *pos_out);
short pix_double_to_long( double pos_in,  bool random_round, long   *pos_out);


 /***********************************\
|       Error Handlings               |
 \***********************************/
/*
  Error message and ID processing.
 */

short pix_put_msg(const char *err_message);
short pix_put_status(short status);

short pix_get_errstatus(void);
short pix_get_errmsg( char *);
short pix_get_error(char *err_message);

short pix_clear_msg(void);
bool  pix_noerror(void);
bool  pix_ignore_error(short status);
void pix_show_status_message(void);
void  pix_prt_errmsg(void);

#ifdef  __cplusplus
}
#endif

#endif /* _PIXLIB_H_ */
