/*                                                                
**  Copyright (C) 1998-2002,2005,2007  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


#ifndef PSF_LIB_H
#define PSF_LIB_H

#include "ascdm.h"
#include <dslib.h>
#include "liberr.h"

typedef struct _psfVar psfVar;

extern int psfGetInit(psfVar** col_ptr,
		      char* psfTable,
		      dsErrList* errList
		      );

extern int psfGetFree( psfVar* cols );

extern int psfGetRadius(psfVar* cols,
			double* energy_in,
			double* theta_in,
			double* phi_in,
			double* ecf_in,
			long nelems,
			dsErrList* errList,
			double* psfSize
			);

extern int psfGetECF(psfVar* cols,
		     double* energy_in,
		     double* theta_in,
		     double* phi_in,
		     double* radius,
		     long nelems,
		     dsErrList* errList,
		     double* psfFrac
		     );

extern float psfCalcSize( 
		  char *mission,     /* i: mission name */
		  char *detector,    /* i: detector/instrument name */
		  char *grating,     /* i: grating or sub-detector */
		  int order,         /* i: grating order */
		  char *direction,   /* i: grating direction (dispersion/cross-d) */
		  float **spectrum,  /* i: energy spectrum to to weight by */
		  int n_spectrum,    /* i: no. energies in spectrum array */
		  float theta,       /* i: off axis angle, arcmin */
		  float defocus,     /* i: detector defocus, mm */
		  float ee_fraction, /* i: return size at this EE fraction */
		  char *psfTable,    /* i: calib file name */
		  dsErrList *errList /* o: error list for psf lib*/
		  );

extern float psfCalcSizeArray( 
		  char *mission,     /* i: mission name */
		  char *detector,    /* i: detector/instrument name */
		  char *grating,     /* i: grating or sub-detector */
		  int order,         /* i: grating order */
		  char *direction,   /* i: grating direction (dispersion/cross-d) */
		  float **spectrum,  /* i: energy spectrum to to weight by */
		  int n_spectrum,    /* i: no. energies in spectrum array */
		  float theta,       /* i: off axis angle, arcmin */
		  float defocus,     /* i: detector defocus, mm */
		  float ee_fraction, /* i: return size at this EE fraction */
		  char *psfTable,    /* i: calib file name */
		  dsErrList *errList,/* o: error list for psf lib*/
		  float *sizeArray   /* o: size vs. energy array */
		  );




extern float psfCalcFrac( 
		  char *mission,     /* i: mission name */
		  char *detector,    /* i: detector/instrument name */
		  char *grating,     /* i: grating or sub-detector */
		  int order,         /* i: grating order */
		  char *direction,   /* i: grating direction (dispersion/cross-d) */
		  float **spectrum,  /* i: energy spectrum to to weight by */
		  int n_spectrum,    /* i: no. energies in spectrum array */
		  float theta,       /* i: off axis angle, arcmin */
		  float defocus,     /* i: detector defocus, mm */
		  float radius,      /* i: return EE fraction at this radius */
		  char *psfTable,    /* i: calibration file name */
		  dsErrList *errList /* o: error list */
		  );

extern float psfCalcFracArray( 
		  char *mission,     /* i: mission name */
		  char *detector,    /* i: detector/instrument name */
		  char *grating,     /* i: grating or sub-detector */
		  int order,         /* i: grating order */
		  char *direction,   /* i: grating direction (dispersion/cross-d) */
		  float **spectrum,  /* i: energy spectrum to to weight by */
		  int n_spectrum,    /* i: no. energies in spectrum array */
		  float theta,       /* i: off axis angle, arcmin */
		  float defocus,     /* i: detector defocus, mm */
		  float radius,      /* i: return EE fraction at this radius */
		  char *psfTable,    /* i: calibration file name */
		  dsErrList *errList,/* o: error list */
		  float *fracArray   /* o: EE fraction vs. energy */
		  );




extern void*      psfNull(char*, ...);
extern int        psfError(char*, ...);

extern float* do_psfmodel(
			  char*    detector,  /* I: user param: detector name   */
			  float    theta,     /* I: off axis angle: elevation, arcmin   */
			  float    phi,       /* I: off axis angle: azimuth, deg     */
			  float**  spectrum,  /* I: 2-column array of energy (keV) */
			                      /*    and counts                  */
			  int      nspec,     /* I: how many rows in spectrum array */
			  char*    psf_table, /* I: user param: table of psf image
						    file names                  */
			  int      verbose,   /* I: user param: verbose mode    */
			  long*    nrows,     /* O: how many rows in returned
						    arrray                      */
			  long*    ncols);     /* O: how many columns in returned
						    array                       */

extern float** psfMakeSpectrum(
                char *spectrumFile,
                int   spectNum,
                char *energyRange,
                float energyStep,
                char *spectrumModel,
                float modelParam,
                int  *numSpec,
                dsErrList *errList
                );
/*
  extern float** psfMakeSpectrum(
  char*   spec_file,       /K I: spectrum file name K/
  char*   energy,          /K I: user param: energy or energy
  range in keV                K/
  float   step,            /K I: user param: interval in keV
  between energies in range   K/
  char*   spectral_model,  /K I: model string K/
  float   model_param,     /K I: model parameter K/
  int*    nspec);           /K O: how many rows in returned
  array                       K/  
  */


extern void psfFreeSpectrum( float **spectrum );



extern float psfCalcSizeThetaArray( 
		  char *mission,     /* mission name */
		  char *detector,    /* detector/instrument name */
		  char *grating,     /* grating or sub-detector */
		  int order,         /* grating order */
		  char *direction,   /* grating direction (dispersion/cross-d) */
		  float **spectrum,  /* energy spectrum to to weight by */
		  int n_spectrum,    /* no. energies in spectrum array */
		  float *thetaArray, /* off axis angle, unit? (degrees, arcmin/sec?) */
		  int nTheta,
		  float defocus,     /* detector defocus, mm */
		  float ee_fraction, /* return size at this EE fraction */
		  char *psfTable,    /* file name for calibration PSF data */
		  dsErrList *errList,/* error list */
		  float *sizeArray   /* array of sizes vs. energy */
		  );

extern float psfCalcFracThetaArray( 
		  char *mission,     /* mission name */
		  char *detector,    /* detector/instrument name */
		  char *grating,     /* grating or sub-detector */
		  int order,         /* grating order */
		  char *direction,   /* grating direction (dispersion/cross-d) */
		  float **spectrum,  /* energy spectrum to to weight by */
		  int n_spectrum,    /* no. energies in spectrum array */
		  float *thetaArray, /* off axis angle, arcmin */
		  int nTheta,
		  float defocus,     /* detector defocus, mm */
		  float radius, /* return size at this EE fraction */
		  char *psfTable,    /* file name for calibration PSF data */
		  dsErrList *errList,/* error list */
		  float *fracArray   /* return array of fractions vs. energy */
		  );

/******** new psfmodel functions: ********************************************/

extern int psf_set_lib_file (char *file);

extern int psf_calc_fraction(
                   float         x,            /* I; x-coordinate       */
                   float         y,            /* I; y-coordinate       */
                   dmBlock      *events_block, /* I; image block        */
                   float        *energy_list,  /* I; list of energies   */
                   unsigned int  num_energies, /* I; number of energies */
		   char         *pix_geom_par, /* I; pixlib geom par    */
                   float        *psf_fractions /* O; list of fractions  */
                            );

extern int psf_calc_fraction_arf(
                   char         *file_name,    /* I; PSF library file   */
                   float         x,            /* I; x-coordinate       */
                   float         y,            /* I; y-coordinate       */
                   dmBlock      *bin_block,    /* I; image block        */
                   float        *energy_list,  /* I; list of energies   */
                   unsigned int  num_energies, /* I; number of energies */
		   char         *pix_geom_par, /* I; pixlib geom par    */
                   float        *psf_fractions /* O; list of fractions  */
                                );

extern int psf_calc_one_frac_arf(
                      float         f,            /* I; defocus            */
                      float         energy,       /* I; energy             */
                      float         x,            /* I; x-coordinate       */
                      float         y,            /* I; y-coordinate       */
                      float         order,        /* I; order              */
                      dmBlock      *bin_block,    /* I; image block        */
                      char         *pix_geom_par, /* I; pixlib geom par    */
                      char         *file_name,    /* I; PSF library file   */
                      regRegion    *region,       /* I; image region       */
                      float        *psf_fraction  /* O; PSF fraction       */
                                );

extern int psfGetASCFITPSF(
                       float      binspa_X,         /* I; PSF binning - x dir. */
                       float      binspa_Y,         /* I; PSF binning - y dir. */
                       int        sizeout_X,        /* I; PSF size - x dir.    */ 
                       int        sizeout_Y,        /* I; PSF size - y dir.    */
                       float      coord_F,          /* I; desired focus        */
                       float      coord_E,          /* I; desired energy       */
                       float      coord_X,          /* I; desired x-coordinate */
                       float      coord_Y,          /* I; desired y-coordinate */
                       float      coord_O,          /* I; desired order        */
                       int        pix_Points,       /* I; points per pixel     */
                       int        verbose,          /* I; debug print level    */
                       dmBlock   *asc_Block,        /* I; image block          */
                       char      *fits_File,        /* I; PSF file name        */
		       bool       clobber,          /* I: overwrite file?      */
                       char      *pixlib_geom_File, /* I; pixlib geom par file */
                       char      *out_File,         /* O; PSF image file       */
                       char      *out_PSF_File,     /* O; PSF lib data file    */
                       float    **psf_Array,        /* O; PSF image            */
                       int       *psf_Size_X,       /* O; PSF image x-size     */
                       int       *psf_Size_Y,       /* O; PSF image y-size     */
                       float     *det_X,            /* O; detx-coordinate      */
                       float     *det_Y             /* O; dety-coordinate      */
                          );

extern int psfCalcFracPSF(
                          float         f,
                          float        *energy_list,
                          float         x,
                          float         y,
                          float         order,
                          unsigned int  num_energies,
                          dmBlock      *bin_block,
			  char         *pix_geom_par,
                          char         *file_name,
                          char         *out_file,
                          regRegion    *region,
                          float        *psf_fractions
                         );

extern int psf_calc_one_frac(
                      float         f,            /* I; defocus            */
                      float         energy,       /* I; energy             */
                      float         x,            /* I; x-coordinate       */
                      float         y,            /* I; y-coordinate       */
                      float         order,        /* I; order              */
                      dmBlock      *bin_block,    /* I; image block        */
                      char         *pix_geom_par, /* I; pixlib geom par    */
                      char         *file_name,    /* I; PSF library file   */
                      char         *out_file,     /* I; output file name   */
                      regRegion    *region,       /* I; image region       */
                      float        *psf_fraction  /* O; PSF fraction       */
                            );

extern int psf_get_all_energies(
                  char   *fits_file,      /* I; PSF library file          */
                  int    *tbl_energies,   /* O; total number of energies  */
                                          /*    in PSF library file table */
                  float **tbl_energy_list /* O; pointer to array of table */
                                          /*    energy values             */
                               );

extern int psf_resize_image (
                     int     in_size_X, /* I; input image size in x dir. */
                     int     in_size_Y, /* I; input image size in y dir. */
                     float*  in_Image,  /* I; input image                */
                     int     out_size_X, /* I; output image size in x dir. */
                     int     out_size_Y, /* I; output image size in y dir. */
                     float*  out_Image   /* O; input image                 */
                            );

extern int psf_frac_interpolate(
           int           tbl_energies,    /* I; total number of energies */
                                          /*    in energy table          */
           float        *tbl_energy_list, /* I; list of all energies in  */
                                          /*    energy table             */
           float        *tbl_fractions,   /* I; list of fractions for    */
                                          /*    tbl_energy_list          */
           unsigned int  in_energies,     /* I; total number of input    */
                                          /*    energies                 */
           float        *in_energy_list,  /* I; list of input energies   */
           float        *psf_fractions    /* O; list of interpolated     */
                                          /*    fractions for input      */
                                          /*    energies                 */
                               );

extern int psfUseASCFITFile(
                             char   *fits_file,
                             float   x,
                             float   y
                           );

extern int psfFreePSF ( float *psf );

extern int psflib_init (void);

extern int psflib_close (void);

#endif

