// 
//  Copyright (C) 1999-2001,2003,2007-2009,2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H**************************************************************************

   FILE NAME:           @(#)rmf_string.hh

   DEVELOPMENT:         Chandra X-ray Science Center fef library
                        hhe@cfa.harvard.edu (initial version)

   DESCRIPTION:         Header file 

* REVISION HISTORY:
 
        Ref. No.        Date         
        --------        ----
        1.1             
                        

*H**************************************************************************/
#ifndef _RMF_STRING_HH_
#define _RMF_STRING_HH_

#include <cstdio>
#include <cstdlib>
#include <string>
#include <vector>
using namespace std;


#if defined (npos)
  //#define npos (size_t)-1
#else
  static const size_t npos = size_t(-1);
#endif


#include <string.h> // for strlen
#include <stdlib.h> // for strtod 

//=============================
// Class rmfString
//=============================
class rmfString
{
  
public:
  rmfString(){d_ldelim = d_rdelim = d_token = ' ';};
  rmfString(string in_str)
    { d_string = in_str; d_ldelim = d_rdelim = d_token = ' ';};
  ~rmfString(){};
  
  void SetToken(string s_token) {d_token = s_token;};
  void SetToken( char *s_token) {d_token = s_token;};
  vector<float> GetFloat(string in_str)
      {  d_string = in_str; return GetFloat();};
  vector<short>  GetInteger(string in_str)
      {d_string = in_str; return GetInteger();};
  vector<float> GetFloat();
  vector<short>  GetInteger();

  string CatStrings(char* str1, char *str2);
  string UpperCase();
  string UpperCase(const char *str){d_string = (char*)str; return UpperCase(); };
  string UpperCase(string str){d_string = str; return UpperCase(); };
  string LowerCase();
  string LowerCase(string );
  void SetRightDelim(string delim){d_rdelim = delim;};
  void SetLeftDelim(string delim){ d_ldelim = delim;};
  vector<string> GetSubStrings();

  vector<string> GetSubStrings_1delim(string, string delim);
  vector<string> GetSubStrings_1delim(string _str, const char *delim){
    return  GetSubStrings_1delim(_str, string(delim) );};
  vector<string> GetSubStrings_1delim(string _str, char *delim){
    return  GetSubStrings_1delim(_str, string(delim) );};
  /*
  vector<string> GetSubStrings_1delim(string _str, char _delim){
    char delim[2]; delim[0]=_delim;
    return  GetSubStrings_1delim( _str, string(delim) );};
    */

  vector<string> GetSubStrings(string, string delim_l, string delim_r);
  vector<string> GetSubStrings(string in_string, char _delim_l, char _delim_r)
  { char delim_l[2], delim_r[2]; delim_l[0]=_delim_l;delim_r[0]=_delim_r;
  return GetSubStrings(in_string, string(delim_l), string(delim_r) ); };
  string GetSubString(string in_string, string delim_l, string delim_r);
  /*
  string GetSubString(string in_string, char _delim_l, char _delim_r)
    { char delim_l[2], delim_r[2]; delim_l[0]=_delim_l;delim_r[0]=_delim_r;
    return GetSubString(in_string, string(delim_l), string(delim_r) ); };
    */

  string GetSubString();
  bool IsWhiteSpace(const char *_str);
  bool IsWhiteSpace(){ 
    return IsWhiteSpace( d_string.c_str() );};

  vector<double> GetDouble();

  char *Double2Str(double cnt);
  double Str2Double();
  double Str2Double(const char *s);
  long Str2Long();
  long Str2Long(const char *);
  char *Long2Str(long);
  float Str2Float();
  float Str2Float(const char *);
  char *Float2Str(float);
  size_t TrimWhiteSpace_back(string &_str);
  size_t TrimWhiteSpace_front(string &_str);
  size_t TrimWhiteSpace(string &_str);
  void prepend(char *_str){  
    if( string(_str).length() == 0 ) return;
    d_string = string(_str);  d_string += d_string;
  }

private:
  string d_string;
  string d_ldelim;
  string d_rdelim;
  string d_token;
};

#endif // _RMF_STRING_HH_




