// 
//  Copyright (C) 1998-2003,2007,2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:    @(#)write_rmf.hh
 
* DEVELOPEMENT: Chandra X-ray Science Center fef library
                hhe@cfa.harvard.edu (Initial version)
 
* DESCRIPTION:  Header file

 
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             12/97
        1.0.1           10/98
 
*H****************************************************************************/

#ifndef _WRITE_RMF_HH_
#define _WRITE_RMF_HH_

#include "read_fef.hh"

extern "C" {
 #include "ascdm.h"
 #include "fitsio.h"
}

//
// abstract class
//

//class RMFFormat : public RMFKeywords
class RMFFormat : public RMFReadFEF
{

public:

  RMFFormat():  RMFReadFEF(), d_dmDS(NULL), d_dmBK(NULL) 
  { InitArray(); };
  RMFFormat(dsErrList* _errLst, FILE *_log, short _verb=1):RMFReadFEF(_errLst, _log, _verb),
    d_dmDS(NULL),
    d_dmBK(NULL)
  { InitArray(); };
  virtual ~RMFFormat(){ FreeArray();};

  virtual RMFGeneral::ERROR RMFHeader()= 0;
  virtual RMFGeneral::ERROR RMFData()= 0;
  virtual RMFGeneral::ERROR CreateFITSFile() = 0;

  RMFGeneral::ERROR PrimaryHeader();
  RMFGeneral::ERROR EBHeader();
  RMFGeneral::ERROR EBData();
  RMFGeneral::ERROR WriteHeader(char *extn);

  void SetOutputFileName(const char *fname){ d_matrixFile = (char*)fname; };
  void SetOutputFmt(const char* oput_fmt){d_dparam.output_fmt = oput_fmt;}; 
  void SetGriddedAxis(short axis) {d_dparam.gridded_axis = axis;};
  void SetGridsAtAxis(short axis, vector<float>);
  void SetAxisName(short n_axis, string *name);
  void SetBinStep(float *bin_step);
  void SetMatrix(vector<float> *m);
  void SetEBounds(vector<float> eb);
  void setOutputChannel(vector<long> outChans){d_outchan=outChans;};

  RMFGeneral::ERROR BinMatrix();
  RMFGeneral::ERROR BinEBound();
  RMFGeneral::ERROR ReadFEFHeader(vector<string> );
  RMFGeneral::ERROR WriteSubspace();
  vector<string> GetExtName(){return d_extName; };

protected:
  void SetEBColumns(char**, char**);
  RMFGeneral::ERROR GetColNameAndUnitAtAxis(short axis, char *ttype[], char *tunit[]);
  RMFGeneral::ERROR BinGridsByStep(float, vector<float> &, char='s');
  RMFGeneral::ERROR RegriddingByStep(float, vector<float> &);

  RMFGeneral::ERROR UpdateEBHeader();
  RMFGeneral::ERROR ReadyForData();
  RMFGeneral::ERROR arrangeOutArrays(long ngrp, long *Fchan,  long *Nchan, 
				     long *tot, float* values);


  char *d_matrixFile;           // output file of rmf matrix
  bool d_lowThresh;     // true for nonzero lowThreshold
  vector<long> d_outchan;

typedef struct data_param_s
{
  short dimension;
  string *axis_name;           // axis names for n dimension
  vector<float> *unbin_grid_space; // unbinned grid range vectors  along n dimension
  vector<float> *grid_space;   // binned grid range vectors along n dimension
  vector<float> *data_space;   // data value vectors along n dimension (unbinnedand then binned
  short         gridded_axis;  // values are griddled at the axis.
  string output_fmt;           // output format: legacy or cxc

  long          siz_ngp,       // Max # elements in F_CHAN & N_CHAN arrays 
                siz_mat;       // Max # elements in MATRIX array 
  short         var_lnth;      // 1 for variable length array, 0 for fixed 
            
  long           *n_grp; 
  vector<long>   *f_chan; 
  vector<long>   *n_chan;
  float          *bin_step;             // bin factor

  long          gaxis_bin;   // compressed bin size along the gridded aixs  
  vector<float> eb_array;    // Full EBOUNDS array

  char fefFile[MAX_STRING_LEN];

} data_param;
data_param d_dparam;

  vector<string> d_extName;
  dmDataset    *d_dmDS;
  dmBlock      *d_dmBK;
  RMFGeneral::ERROR   d_status;

private:

  RMFGeneral::ERROR GetEffPhaRange();
  RMFGeneral::ERROR SortMatrix();
  RMFGeneral::ERROR FindDetChans();
  RMFGeneral::ERROR GetNgrpFchanNchan();
  void InitArray();
  void FreeArray();

  const char *const className() { return ("RMFFormat"); };

};

//============================================================
//
// Class RMFLegacy
// instance class of the abstrct class above with an extension
//
//============================================================
class RMFLegacy: public RMFFormat
{

public:

  RMFLegacy():RMFFormat(),d_fptr(NULL),d_fstatus(0){};
  RMFLegacy(dsErrList* _errLst, FILE* _log):RMFFormat(_errLst,_log),d_fptr(NULL),d_fstatus(0){};
  virtual ~RMFLegacy(){;};

  RMFGeneral::ERROR WriteEBounds();
  RMFGeneral::ERROR WriteMatrix();
  RMFGeneral::ERROR WriteColumn();

private:

  virtual RMFGeneral::ERROR RMFHeader();
  virtual RMFGeneral::ERROR RMFData();
  virtual RMFGeneral::ERROR CreateFITSFile();


  void  CloseCFitsFile();
  void fitsError(long _line);
  void fitsError(int status, long _line) {d_fstatus = status; fitsError(_line); d_fstatus = 0;};

  const char *const className() { return ("RMFLegacy"); };

  fitsfile *d_fptr;
  int d_fstatus;  // for fits error status only


     
};
//==============================================================
//
// Calss RMFCXC
//
//
// instance class of the abstrct class above with an extension
//==============================================================
class RMFCXC: public RMFFormat
{

public:

  RMFCXC():RMFFormat(){};
  RMFCXC(dsErrList* _errLst, FILE* _log):RMFFormat(_errLst, _log){};
  virtual ~RMFCXC(){};

  RMFGeneral::ERROR WriteMatrix();
  RMFGeneral::ERROR WriteEBounds();

private:

  virtual RMFGeneral::ERROR RMFHeader();
  virtual RMFGeneral::ERROR RMFData();

  RMFGeneral::ERROR CreateFITSFile();
  RMFGeneral::ERROR SetMatrixStruc();
  RMFGeneral::ERROR WriteMatrixStruct();

  const char *const className() { return ("RMFCXC"); };

  typedef struct resp_matrix_s
  {
    long   cellnaxi;            // number of matrix coordinates
    vector<string> celltypn;   // name of the n^th matrix coord
    vector<string> celluntn;   // units of the n^th coord
    vector<long>   cellnaxn;    // number of bins in the n^th coord
    vector<float>  cellminn;    // minimum value  of the n^th coord
    vector<float> cellmaxn;     // maximum value  of the n^th coord
    short  ncols;               // number of columns of the matrix table
    vector<string> colname;    // column names
  } resp_matrix;
  resp_matrix d_rmatrix;

      
};
//=======================================
//
// Class RMFGeneral
//
//=======================================
class WriteRMF: public RMFGeneral
{

public:
  WriteRMF():RMFGeneral(), d_rmflgc(NULL, NULL), d_rmfcxc(NULL, NULL) {};
  WriteRMF(dsErrList* _errLst, FILE* _log, short verb):RMFGeneral(_errLst,_log,verb), d_rmflgc(_errLst,_log), d_rmfcxc(_errLst,_log){};
  virtual ~WriteRMF(){};

  RMFGeneral::ERROR WriteRMFs(vector<long> outchan);

  void SetInput(vector<double>, vector<double>, vector<double> );
  void SetInput(vector<float>, vector<float>, vector<float> );
  void SetOutputFileName(const char *fname){ d_outfile = (char*)fname;};
  void SetOutputFmt(const char* oput_fmt);

  void SetGridsAtAxis(short axis, vector<float>);
  void SetAxisName(short n_axis, string *name);
  void SetGriddedAxis(short axis);
  void SetBinStep(float *bin_step);
  void SetMatrix(vector<float> *m);
  void SetEBounds(vector<float> eb);

  RMFGeneral::ERROR SetKeywordValue_f(char *keywd, float val);
  RMFGeneral::ERROR SetKeywordValue_s(char *keywd, char *val);


private:
  const char *const className() { return ("WriteRMF"); };
  RMFLegacy d_rmflgc;
  RMFCXC d_rmfcxc;
  string d_outfmt;
  string d_outfile;       // name of output matrix file
};
#endif //_WRITE_RMF_HH_
