#!/usr/bin/env perl
#
#############################################################
#
# REVISION HISTORY:
#  
#  Ref. No.        Date     Auth.
#  --------        ----     ----
#
#############################################################
# Turn off buffering
select(STDOUT); $| = 1;

my $tool = "scconv";
my $testexe = "scconv_test";

use File::Basename;

printf("\t %s REGRESSION TEST\n", uc($tool));

#set env variables
$grade = "FAIL";
$USER  = $ENV{"USER"};
$ENV{"ASCDS_VERSION"}="TEST";

#Define base directories
($indir, $outdir, $savdir, $logdir) = &getTestdirs;

#Add specific paths to data for this test
$indir  .= "/test_${tool}";
$outdir .= "/${tool}";
$savdir .= "/${tool}";

#Get the date for log name
$date=`date +%d%b%Y_%H:%M:%S`;
chop($date);

#Define files
$logfile="${tool}_log.${date}";
$outverbose = "${tool}_verbose.txt";
@outfiles = ( $outverbose );

#Directory setup
if (! -d $outdir ) 
{
  system("mkdir -p $outdir");
}

#Clear out old output files
system("rm -f ${outdir}/* ");

#Open log file to contain run dump
open(SAVOUT,">&STDOUT");
open(STDOUT, "> ${logdir}/${logfile}");

#----------------------------------------------------------------------
# Find Test executable.
#----------------------------------------------------------------------
$cwd = dirname($0);
$texedir = "./";
if ( ! -e "${cwd}/scconv/${testexe}" )
{
  print "ERROR: Test Executable not found. \n";
}
else
{
  $texedir = "${cwd}/scconv";
}


#----------------------------------------------------------------------
# Run Test(s)
#----------------------------------------------------------------------
#
# 
print("\tRunning test ....\n");

$comstr = "${texedir}/${testexe}";
#print "\n\nCOMSTR=$comstr \n";
system($comstr);


#----------------------------------------------------------------------
# Evaluate Test(s)
#----------------------------------------------------------------------
# Close up log file
close(STDOUT);
open(STDOUT,">&SAVOUT");

# Filter/Transfer the log file to the output directory
&Filter_Text_File( "${logdir}/${logfile}", "${outdir}/${outverbose}", NULL );

# Check output files
print " Tests complete\n";
print " Log file = $logfile \n";
print " Checking output files...\n";
print "   $outdir\n";

$anyprobs = 0;
foreach $file ( @outfiles )
{
  if ( ($file =~ /\.txt$/ ) || ($file =~ /verbose/) )
  {

    unless( -e "${outdir}/${file}" )
    {
	printf "File not created: $file \n";
	$anyprobs++; 
	next;
    }

    $ndiffs = &Diff_Files( "${outdir}/${file}","${savdir}/${file}");

    # Check if differences were found
    if ( $ndiffs == -99 )
    {
	# did not find ndiffs... some sort of trouble
	print "- TROUBLE?\n";
	$anyprobs++ ;
    }
    elsif ( $ndiffs > 0 )
    {
	printf ("%33s ... DIFFERS\n", $file );
	$anyprobs++ ;
    }
    else
    {
	printf ("%33s ... OK\n",$file);
    }

  } # End if file should be checked 
} # End file loop

if ( $anyprobs == 0 ){ $grade = "PASS"; }
print STDOUT "\n\t${tool} : $grade \n";

# If PASS, Move current results into save directory
if ( $grade eq "PASS") 
{
  unlink("${logdir}/${logfile}");
  foreach $file ( @outfiles )
  {
    unlink("${outdir}/${file}");
  }
  system("rmdir ${outdir}");
}


#----------------------------------------------------------------------
# SUBROUTINES
#----------------------------------------------------------------------
#================================================================================
sub getTestdirs
{

  local($indir, $outdir, $savdir, $logdir);

  $user = $ENV{'USER'};

  # Define directories for input/output
  $indir = $ENV{'TESTIN'};
  if ( $indir eq "" ) {
      die "getTestdirs could not find environment variable for TESTIN\n";
  }

  $outdir = $ENV{'TESTOUT'};
  if ( $outdir eq "" ) {
      die "getTestdirs could not find environment variable for TESTOUT\n";
  }

  $savdir = $ENV{'TESTSAV'};
  if ( $savdir eq "" )
  {
      die "getTestdirs could not find environment variable for TESTSAV\n";
  }

  $logdir = $ENV{'TESTLOG'};
  if ( $logdir eq "" )
  {
      die "getTestdirs could not find environment variable for TESTLOG\n";
  }

  $dirlist[0] = $indir;
  $dirlist[1] = $outdir;
  $dirlist[2] = $savdir;
  $dirlist[3] = $logdir;

  return(@dirlist);
}
#================================================================================
sub Filter_Text_File
{
  local( $inFile, $outFile, *userFilt ) = @_ ;

  # If you need filter capabilities see mta_perlutils.pl function content.
  $comstr = "cp $inFile $outFile";
  system( $comstr );

}
#==================================================================
sub Diff_Files
{
    local( $FileA, $FileB ) = @_ ;
    local( $ndiff );

    #Check that both files exist
    unless( ( -e "$FileA" ) && ( -e "$FileB" ) )
    {
         return( -99 );
    }
    
    $ndiff = 0;
    open(TSTHND,"diff ${FileA} ${FileB} |");
    while ( $line = <TSTHND> )
    {
         $ndiff++;
    }
    close(TSTHND);
    return($ndiff);
}
