#!/bin/sh

TESTID=ds9-smoke001
export TESTID

#
# Replace with variable via configure
#
ASCDS_QUIET=1; export ASCDS_QUIET
ASCDS_OVERRIDE=1; export ASCDS_OVERRIDE
. $ASCDS_INSTALL/test/smoke/bin/setup.sh

#
# Setup test env
#
TESTDIR=$ASCDS_INSTALL/test/smoke/data; export TESTDIR
if [ "$ASCDS_SMOKE_DIR" = "" ]; then
   OUTDIR=$ASCDS_WORK_PATH/smoke.${LOGNAME}/$TESTID; export OUTDIR
else
   OUTDIR=$ASCDS_SMOKE_DIR/smoke.${LOGNAME}/$TESTID; export OUTDIR
fi
SAVEDIR=$TESTDIR/; export SAVEDIR

INDIR=$TESTDIR

OUTFILE=${OUTDIR}/${TESTID}.txt

if test -d $OUTDIR
then
   \rm -rf $OUTDIR
fi
mkdir -p $OUTDIR

#
# Run command
#
printf "Running test $TESTID "

if [ -z "${DISPLAY}" ] ; then
  echo "... SKIP"
  echo "   Test ${TESTID} skipped- DISPLAY not set" > $OUTFILE
  exit 99;
fi

if [ ! -f "${ASCDS_IMAGER_PATH}/ds9" ] ; then
   printf "... FAIL\n"
   printf "\n    ${ASCDS_IMAGER_PATH}/ds9 is not found.\n" > $OUTFILE
   exit 1 
fi

printf "."
ds9="ds9smoke$$"
ds9 -prefs no -title $ds9 $ASCDS_INSTALL/test/smoke/data/acisf04425_000N003_evt1.fits &
sleep 3
n=1
while test $n -lt 180
do
  if test `xpaaccess $ds9` = yes
  then
     break;
  fi
  sleep 2
  n=`expr $n + 1`
done

xpaset -p $ds9 bin about 3800 3800 2>/dev/null
xpaset -p $ds9 bin factor 2 2>/dev/null
xpaset -p $ds9 scale log 2>/dev/null
xpaset -p $ds9 cmap b 2>/dev/null
xpaset -p $ds9 contour yes 2>/dev/null
xpaset -p $ds9 contour limits 1 100 2>/dev/null
xpaset -p $ds9 contour smooth 5 2>/dev/null
xpaset -p $ds9 contour nlevels 6 2>/dev/null
\rm -f $OUTFILE
xpaset -p $ds9 contour save $OUTFILE wcs fk5 2>/dev/null
xpaset -p $ds9 quit 2>/dev/null
touch $OUTFILE

#
# Run checks on the output files.
#
state=0
printf "."
rm -rf $OUTDIR/tol
echo "col1=range(1e-5)" > $OUTDIR/tol
echo "col2=range(1e-5)" >> $OUTDIR/tol
rm -rf $OUTDIR/file1
cat ${OUTFILE} | grep "^ " > $OUTDIR/file1
rm -rf $OUTDIR/file2
cat ${SAVEDIR}/${TESTID}.txt | grep "^ " > $OUTDIR/file2

dmdiff $OUTDIR/file1 $OUTDIR/file2 tolfile=$OUTDIR/tol >> $OUTDIR/diff.log 2>&1
if test $? != 0
then
   state=1
fi

printf ". "
rm -rf $OUTDIR/tol $OUTDIR/file1 $OUTDIR/file2

#
# Exit condition
#
if test $state = 0
then
  echo "PASS"

  if echo "$OUTDIR" | grep "$TESTID" >/dev/null
  then
     rm $OUTDIR/${TESTID}.*
     rm $OUTDIR/diff.log
     rmdir $OUTDIR
  fi
else
  echo "FAIL"
fi

exit $state
