#!/bin/sh

TESTID=ipython-smoke002
export TESTID

#
# Replace with variable via configure
#
ASCDS_QUIET=1; export ASCDS_QUIET
ASCDS_OVERRIDE=1; export ASCDS_OVERRIDE

#
# Setup test env
#
TESTDIR=$ASCDS_INSTALL/test/smoke/data; export TESTDIR
if [ "$ASCDS_SMOKE_DIR" = "" ]; then
   OUTDIR=$ASCDS_WORK_PATH/smoke.${LOGNAME}/$TESTID; export OUTDIR
else
   OUTDIR=$ASCDS_SMOKE_DIR/smoke.${LOGNAME}/$TESTID; export OUTDIR
fi
SAVEDIR=$TESTDIR/; export SAVEDIR

INDIR=$TESTDIR

OUTFILE=${OUTDIR}/${TESTID}.txt
if test -d $OUTDIR
then
   \rm -rf $OUTDIR
fi
mkdir -p $OUTDIR

#
# Run command
#
printf "Running test $TESTID "

# skip test is sherpa is not installed 
if test -f ${ASCDS_INSTALL}/bin/sherpa
then
   state=0
   if [ -z "${DISPLAY}" ] ; then
     echo "... SKIP"
     echo "   Test ${TESTID} skipped- DISPLAY not set" > $OUTFILE
     exit 99;
   fi
else
  echo "... SKIP"
  echo "   Test ${TESTID} skipped- sherpa not found in CIAO installation" > $OUTFILE
  exit 99;
fi


# Save the current ipython directory 
if [ "x${IPYTHONDIR}" != "x" ] ; then
    REALIPYTHONDIR="${IPYTHONDIR}"
fi

# Set up temporary sherpa ipython profile
IPYTHONDIR="${ASCDS_WORK_PATH}/.ipython-ciao-smoke"
export IPYTHONDIR
if test -d ${IPYTHONDIR}
then
   \rm -rf ${IPYTHONDIR}
fi
ciao_app_setup.sh sherpa .sherpa.rc

printf "."
ipython --no-pprint --profile sherpa -c "print('%s %s %s' % (fit_bkg.__name__, ignore2d.__name__, plot_fit_resid.__name__), file=open('$OUTFILE','w'))" > ${OUTFILE}_err 2>&1
state=$?
printf "."

#
# Run checks on the output files.
#
diff "$OUTFILE" "$SAVEDIR/${TESTID}.txt" >> $OUTDIR/diff.log 2>&1
if test $? != 0
then
   state=1
fi
printf ". "

#
# Exit condition
#
if test $state = 0
then
  echo "PASS"

  if echo "$OUTDIR" | grep "$TESTID" >/dev/null
  then
     rm $OUTFILE
     rm ${OUTFILE}_err
     rm $OUTDIR/diff.log
     rmdir $OUTDIR
  fi
else
  echo " " >> $OUTDIR/diff.log
  echo "- - - - - - - - - - - - - - - - - - - -" >> $OUTDIR/diff.log
  echo "     $TESTID  stderr/sdtout " >> $OUTDIR/diff.log
  echo "- - - - - - - - - - - - - - - - - - - -" >> $OUTDIR/diff.log
  echo " " >> $OUTDIR/diff.log
  cat ${OUTFILE}_err >> $OUTDIR/diff.log
  echo "FAIL"
fi

# Remove the temporary sherpa ipython profile
\rm -rf ${IPYTHONDIR}
export IPYTHONDIR=${REALIPYTHONDIR}

exit $state
