#!/bin/bash

export testfiles=""

add_tests()
{
    tests_to_add="$(ls "${ASCDS_INSTALL}"/test/smoke/bin/*${1}*.sh 2>/dev/null | grep -v -e "run_\|runtime\|_template\|setup")"
    
    if [ "${tests_to_add}x" == "x" ] ; then
        echo "Could not find tests that include '${1}'"
        exit 1
    else
        #Post-pend with a space if this is set
        testfiles=$(echo -e "${testfiles}${testfiles:+\n}${tests_to_add}")
    fi
}

# Check for optional argument to specify a subset of
# tests to run by ID pattern match.
while [ "$#" -ne 0 ]
do
    if [ "${1}" == "all" ] ; then
        add_tests "-smoke"
    else
        add_tests "${1}"
    fi 
    shift
done

if [ "${testfiles}x" == "x" ] ; then
    add_tests "-smoke"
fi

# get number of tests in file list- expect tests to end in 'smoke###.sh' 
CNT=($(echo "$testfiles" | wc -l))

if [ "$CNT" = "0" ]; then
   echo " FAILED: the test file contains no entries"
   exit 0
elif [ "$CNT" = "1" ]; then
   echo " Requested test suite includes $CNT test" 
else
   echo " Requested test suite includes $CNT tests" 
fi

# move into the smoke test directory 
cd "${ASCDS_INSTALL}"/test/smoke/bin

NDX=1
PASSES=0
FAILURES=0
SKIPS=0

#Remove the current smoke test work dir if it exists
if [ "${ASCDS_WORK_PATH}x" != "x" ] && [ -d "${ASCDS_WORK_PATH}/smoke.${LOGNAME}" ]; then
    rm -rf "${ASCDS_WORK_PATH}/smoke.${LOGNAME}"
fi
#Create the smoke test work dir
mkdir -p "${ASCDS_WORK_PATH}/smoke.${LOGNAME}"
stat="$?"
if [ "${stat}" != 0 ] ; then
    echo "ERROR: Failed to create ${ASCDS_WORK_PATH}/smoke.${LOGNAME}"
    exit 1
fi

# if CIAO_DEBUG env var is 0 - only display pass/fail
#                          1 - display diffs in line
#                          2 - display diffs at end 
ERRLOG=""
if [ "${CIAO_DEBUG}" = "2" ]
then
   ERRLOG="${ASCDS_WORK_PATH}/smoke.${LOGNAME}/.diff_logs"  
   if [ -r ${ERRLOG} ]
   then
      \rm ${ERRLOG}
   fi
fi

# loop through specified tests and run them 

for LINE in ${testfiles}
do
   if echo "$LINE" | grep -v "#"  >/dev/null
   then 
      printf "   [${NDX}/${CNT}] " 
      sh $LINE
      state=$?
      if [ $state -eq 99 ]
      then
         SKIPS=`expr $SKIPS + 1`
	 LINE="`basename $LINE | sed -e 's/\.sh//g'`"
         if [ "${ERRLOG}" != "" ] 
         then 
            echo "         Review ${ASCDS_WORK_PATH}/smoke.${LOGNAME}/${LINE}/${LINE}.txt for details" >> ${ERRLOG}
         elif [ "${CIAO_DEBUG}" != "0" ]
         then
            echo "         Review ${ASCDS_WORK_PATH}/smoke.${LOGNAME}/${LINE}/${LINE}.txt for details"
         fi
      elif [ $state -eq 0 ]
      then
         PASSES=`expr $PASSES + 1`
      else
         # test failed- handle logging based on CIAO_DEBUG setting   
         FAILURES=`expr $FAILURES + 1`
	 LINE="`basename $LINE`"
         ERRFILE=`echo $LINE | sed -e 's/\.sh/\/diff\.log/g'`
         if [ "${ERRLOG}" != "" ] 
         then 
            echo "   Review ${ASCDS_WORK_PATH}/smoke.${LOGNAME}/${ERRFILE} for details" >> ${ERRLOG}
         elif [ "${CIAO_DEBUG}" != "0" ]
         then 
            echo "         Review ${ASCDS_WORK_PATH}/smoke.${LOGNAME}/${ERRFILE} for details"
         fi 
      fi 
      NDX=`expr $NDX + 1`
   fi 
done 

if [ $CNT -eq 1 ]
then 
   echo " $CNT smoke test run: $PASSES PASSED, $FAILURES FAILED, $SKIPS SKIPPED"
else
   echo " $CNT smoke tests run: $PASSES PASSED, $FAILURES FAILED, $SKIPS SKIPPED"
fi

if  [ $PASSES != $CNT ]
then
   if [ "${ERRLOG}" != "" ] 
   then 
      cat ${ERRLOG}
      \rm  ${ERRLOG}
   fi 
   echo ""
   echo "       !! CIAO MAY HAVE FAILED TO INSTALL PROPERLY !!"
   echo " Please check https://cxc.harvard.edu/ciao/bugs/smoke.html"
   echo " for a list of known issues before contacting the Help Desk."
   echo ""
fi 



